@@ -47,19 +47,27 @@ CFSBase::~CFSBase()
 // =======================================================
 int CFSBase::readData(void *cBuffer, QWORD qwOffset, DWORD dwLength)
 {
-  BEGIN;
+   BEGIN;
 
-  int nRes;
+   int nRes;
 
-  nRes = bigFseek(m_fDeviceFile, qwOffset);
-  if (nRes != 0)
-    RETURN_int(-1);
+   errno = 0;
+   nRes = bigFseek(m_fDeviceFile, qwOffset);
+   if (nRes != 0)
+   {
+      showDebug(4, "CFSBase::readData(qwOffset=%llu): bigFseek() error %d=%s\n", qwOffset, errno, strerror(errno));
+      RETURN_int(-1);
+   }
 
-  nRes = fread((char *)cBuffer, dwLength, 1, m_fDeviceFile);
-  if (nRes != 1)
-    RETURN_int(-1);
+   errno = 0;
+   nRes = fread((char *)cBuffer, dwLength, 1, m_fDeviceFile);
+   if (nRes != 1)
+   {
+      showDebug(4, "CFSBase::readData(dwLength=%lu): fread() error %d=%s\n", dwLength, errno, strerror(errno));
+      RETURN_int(-1);
+   }
   
-  RETURN_int(0); // success
+   RETURN_int(0); // success
 }
 
 
@@ -133,7 +141,7 @@ void CFSBase::saveUsedBlocksToImageFile(COptions *options)
         THROW(ERR_READ_BITMAP, i);
 
       // stop at the end of the partition
-      dwCount = min(dwCount, m_header.qwBlocksCount-i);
+      dwCount = my_min(dwCount, m_header.qwBlocksCount-i);
 
       showDebug(3, "CPY: dwCount=%lu, bUsed=%d\n", dwCount, bUsed);
 
@@ -179,7 +187,7 @@ void CFSBase::saveUsedBlocksToImageFile(COptions *options)
 		  showDebug(3, "CHECKING(%lu): check=%lu and CRC=%lu\n", i, dwCheckCount, check.dwCRC);
 	      
 		  check.qwPos = i; 
-                  showDebug(1, "BEFORE WRITING CHK\n");
+                  showDebug(3, "BEFORE WRITING CHK\n");
 		  try { m_image->write((char*)&check, sizeof(CCheck), true); }
 		  catch (CExceptions * excep)
 		    {
@@ -276,11 +284,11 @@ void CFSBase::restoreUsedBlocksToDisk(COptions *options)
   i = 0;
   while (i < m_header.qwBlocksCount)
     {
-      showDebug(1, "i = %lld, m_header.qwBlocksCount = %lld\n", i,
+      showDebug(2, "i = %lld, m_header.qwBlocksCount = %lld\n", i,
         m_header.qwBlocksCount);
 
       m_bitmap.readBitmap(i, &bUsed, &dwCount);
-      dwCount = min(dwCount, m_header.qwBlocksCount-i); // stop at the end of the partition
+      dwCount = my_min(dwCount, m_header.qwBlocksCount-i); // stop at the end of the partition
 
       showDebug(3, "CPY: dwCount=%lu, bUsed=%d\n", dwCount, bUsed);
 
@@ -314,10 +322,11 @@ void CFSBase::restoreUsedBlocksToDisk(COptions *options)
 	      showDebug(3, "CRC = %lu\n", dwCRC);
 
 	      dwCheckCount += dwTemp;
+	      showDebug(3, "dwCheckCount= %ld\n", dwCheckCount);
 
 	      if (dwCheckCount == CHECK_FREQUENCY)
 		{
-                  showDebug(1, "BEFORE READING CHK\n");
+                  showDebug(3, "BEFORE READING CHK\n");
 		  try { m_image->read((char*)&check, sizeof(CCheck), true); }
 		  catch (CExceptions * excep)
 		    {
@@ -334,31 +343,38 @@ void CFSBase::restoreUsedBlocksToDisk(COptions *options)
                      || (check.cMagic[2] != 'K'))
 		    {
 		      showDebug(1, "CHECK-MAGIC error %3s %3s\n", "CHK", check.cMagic);
-		      THROW(ERR_WRONGMAGIC);
-		    }
-	      
-		  if (i != check.qwPos)
-		    {
-		      showDebug(1, "CHECK-POS: old=%llu and new=%llu\n",
-                         check.qwPos, i);
-		      
-		      nRes = g_interface->msgBoxYesNo(i18n("local CRC error"),
-                         i18n("There was a position error in CCheck. "
+		      nRes=g_interface->msgBoxYesNo(i18n("local check error"),
+                         i18n("Impossible to find a local check structure. "
                          "Data may be damaged. Do you want to continue ?"));
 		      if (nRes == MSGBOX_NO)
-			THROW(ERR_CHECK_NUM, i, check.qwPos);
+		        THROW(ERR_WRONGMAGIC);
 		    }
+                  else
+                    {
+		      if (i != check.qwPos)
+    		        {
+		          showDebug(1, "CHECK-POS: old=%llu and new=%llu\n",
+                             check.qwPos, i);
+		     
+		          nRes=g_interface->msgBoxYesNo(i18n("local CRC error"),
+                             i18n("There was a position error in CCheck. "
+                             "Data may be damaged. Do you want to continue ?"));
+		          if (nRes == MSGBOX_NO)
+			    THROW(ERR_CHECK_NUM, i, check.qwPos);
+		        }
 
-		  if (dwCRC != check.dwCRC)
-		    {
-		      showDebug(1, "CHECK-CRC(%lu): old=%lu and new=%lu\n", i, 
-                         check.dwCRC, dwCRC);
+		      if (dwCRC != check.dwCRC)
+		        {
+		          showDebug(1, "CHECK-CRC(%lu): old=%lu and new=%lu\n", 
+                             i, check.dwCRC, dwCRC);
 
-		      nRes = g_interface->msgBoxYesNo(i18n("local CRC error"), i18n("There was a CRC error in CCheck. "
-										    "Data may be damaged. Do you want to continue ?"));
-		      if (nRes == MSGBOX_NO)
-			THROW(ERR_CHECK_CRC, dwCRC, check.dwCRC);
-		    }
+		          nRes=g_interface->msgBoxYesNo(i18n("local CRC error"),
+                             i18n("There was a CRC error in CCheck. "
+                             "Data may be damaged. Do you want to continue ?"));
+		          if (nRes == MSGBOX_NO)
+			    THROW(ERR_CHECK_CRC, dwCRC, check.dwCRC);
+		        }
+                    }
 
                   dwCRC = increaseCrc32(0, NULL, 0);
 		  dwCheckCount = 0;
@@ -367,8 +383,10 @@ void CFSBase::restoreUsedBlocksToDisk(COptions *options)
 		  if (dwStats == 8)
 		    {
 		      // update stats
-		      m_guiRestore->showStats(g_timeStart, m_header.qwBlockSize, dwSavedBlocks,
-					      m_header.qwUsedBlocks,m_header.qwBlocksCount, options->bEraseWithNull);
+		      m_guiRestore->showStats(g_timeStart,
+                         m_header.qwBlockSize, dwSavedBlocks,
+                         m_header.qwUsedBlocks,m_header.qwBlocksCount,
+			 options->bEraseWithNull);
 		      dwStats = 0;
 		    }
 		  dwStats++;
@@ -408,7 +426,7 @@ void CFSBase::restoreUsedBlocksToDisk(COptions *options)
 		  errno = 0;
                   if (!options->bSimulateMode)
 		    {
-		      nRes = fwrite(cZeroBuffer, min(ZERO_BLOCK_SIZE, dwTemp),
+		      nRes = fwrite(cZeroBuffer, my_min(ZERO_BLOCK_SIZE, dwTemp),
                          1, m_fDeviceFile);
 		      if (nRes != 1)
                         THROW(ERR_ZEROING, errno);
@@ -416,7 +434,7 @@ void CFSBase::restoreUsedBlocksToDisk(COptions *options)
                   else
                     showDebug(10, "fwrite simulated\n");
 		  
-		  dwTemp -= min(ZERO_BLOCK_SIZE, dwTemp);
+		  dwTemp -= my_min(ZERO_BLOCK_SIZE, dwTemp);
 		}
 	      dwSavedBlocks += dwCount;
 	    }
@@ -468,7 +486,7 @@ void CFSBase::saveToImage(CImage *image, COptions *options, CSavingWindow *gui)
 
       try { fsck(); }
       catch ( CExceptions * excep )
-        { THROW(ERR_FSCHK); }
+        { throw excep; }
     }
 
   // read bitmap into memory (must be done before writing header)
