@@ -16,6 +16,16 @@
  *                                                                         *
  ***************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+  #include <config.h>
+#endif
+
+#include <unistd.h>
+#include <sys/types.h>
+#ifdef HAVE_SYS_STAT_H
+  #include <sys/stat.h>
+#endif
+
 #include "fs_ext2.h"
 #include "partimage.h"
 #include "imagefile.h"
@@ -41,14 +51,38 @@ void CExt2Part::fsck()
 
   char szCommand[1024];
   int nRes;
+  struct stat fStat;
 
-  SNPRINTF(szCommand, "/sbin/e2fsck -nf -C 0 %s > /dev/null 2>/dev/null", m_szDevice);
-  //snprintf(szCommand, sizeof(szCommand), "/sbin/e2fsck -nf -C 0 %s > /dev/null 2>/dev/null", m_szDevice);
-  //*(szCommand+1023) = '\0';
+// check if /sbin/e2fsck can be executed
+  nRes = stat("/sbin/e2fsck", &fStat);
+  if (nRes == -1)
+    {
+      nRes = g_interface -> askIgnoreNoFschk("/sbin/e2fsck");
+      if (nRes == MSGBOX_CANCEL)
+        THROW(ERR_CANCELED);
+      else
+        RETURN;
+    }
+
+  if ( !S_ISREG(fStat.st_mode) ||
+       !(fStat.st_mode & S_IXOTH) &&
+       ( getgid() != fStat.st_gid || !(fStat.st_mode & S_IXGRP) ) &&
+       ( getuid() != fStat.st_uid || !(fStat.st_mode & S_IXUSR) ) )
+    {
+      nRes = g_interface -> askIgnoreDeniedFschk("/sbin/e2fsck");
+      if (nRes == MSGBOX_CANCEL)
+        THROW(ERR_CANCELED);
+      else
+        RETURN;
+    }
+
+  SNPRINTF(szCommand, "/sbin/e2fsck -nf -C 0 %s > /dev/null 2>/dev/null",
+     m_szDevice);
   
   showDebug(1, "FSCK: [%s]\n", szCommand);
 
   nRes = system(szCommand);
+  showDebug(1, "FSCK: return is %d\n", nRes); 
   if (nRes != 0)
     {	
       nRes = g_interface -> askIgnoreFSError("e2fsck");
@@ -187,24 +221,36 @@ void CExt2Part::readBitmap(COptions *options) // FULLY WORKING
 
   cTempBitmap = new char[((m_info.dwTotalBlocksCount+7)/8)+4096];
   if (!cTempBitmap)
-    goto error_readBitmap;
+  {  
+     showDebug(1, "CExt2Part::readBitmap(): Error 001\n");  
+     goto error_readBitmap;
+  }
 
   // init bitmap size
   nRes = m_bitmap.init(m_header.qwBitmapSize);
   if (nRes == -1)
-    goto error_readBitmap;
+  { 
+     showDebug(1, "CExt2Part::readBitmap(): Error 002\n");  
+     goto error_readBitmap;
+  }
 
   // load group descriptors
   desc = new CExt2GroupDesc[m_info.dwGroupsCount+m_info.dwDescPerBlock];
   if (!desc)
-    goto error_readBitmap;
+  { 
+     showDebug(1, "CExt2Part::readBitmap(): Error 003\n");  
+     goto error_readBitmap;
+  }
 
   // for each descriptor BLOCK (not group descriptor!)
   for (cPtr=(char*)desc, i=0; i < m_info.dwDescBlocks; i++,cPtr+=m_info.dwBlockSize)
     {
       nRes = readData(cPtr, ((QWORD)m_info.dwBlockSize) * ((QWORD)(m_info.dwFirstBlock+1+i)), m_info.dwBlockSize);
       if (nRes == -1)
-	goto error_readBitmap;
+      { 
+	 showDebug(1, "CExt2Part::readBitmap(): Error 004\n");  
+	 goto error_readBitmap;
+      }
     }
 
   dwUsed=0;
@@ -220,9 +266,15 @@ void CExt2Part::readBitmap(COptions *options) // FULLY WORKING
       if (Le32ToCpu(desc[i].bg_block_bitmap))
 	{
 	  // -- read the bitmap block
-	  nRes = readData(cTempBitmap+(i*m_info.dwBlockSize), ((QWORD)m_info.dwBlockSize) * ((QWORD)Le32ToCpu(desc[i].bg_block_bitmap)), m_info.dwBlockSize);
+	   errno = 0;
+	  nRes = readData(cTempBitmap+(i*m_info.dwBlockSize), ((QWORD)m_info.dwBlockSize) * 
+			  ((QWORD)Le32ToCpu(desc[i].bg_block_bitmap)), m_info.dwBlockSize);
 	  if (nRes == -1)
-	    goto error_readBitmap;
+	  {
+	     showDebug(1, "CExt2Part::readBitmap(): Error 005\n");  
+	     showDebug(1, "CExt2Part::readBitmap(): err=%d=%d\n", errno, strerror(errno));  
+	     goto error_readBitmap;
+	  }
 	}
       else
 	{
