@@ -5,7 +5,7 @@
     copyright            : (C) 2000 by Franois Dupoux
     email                : fdupoux@partimage.org
  ***************************************************************************/
-// $Revision: 1.18 $
+// $Revision: 1.25 $
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -66,7 +66,7 @@ int CRestoreMbrWindow::create()
 
   m_radioFull = newtRadiobutton(1, 13, i18n("The whole MBR"), true, NULL);
   m_radioBoot = newtRadiobutton(1, 14, "Only the boot loader", false, m_radioFull);
-  m_radioTable = newtRadiobutton(1, 15, "Only the partition table", false, m_radioBoot);
+  m_radioTable = newtRadiobutton(1, 15, "Only the primary partitions table", false, m_radioBoot);
 
   addButtons();
 
@@ -137,6 +137,8 @@ char *processHardDrive(char *cPtr, newtComponent editPartition)
   cOldPtr = cPtr;
   nPartNum = 1;
 
+  showDebug(9, "decode HD\n");
+
   while (*(cPtr) && *(cPtr+1) && *(cPtr+2) && nPartNum)
     {
       cPtr = decodePartitionEntry(cPtr, &nMajor, &nMinor, &nBlocks, &nPartNum, szDevice);
@@ -167,9 +169,9 @@ char *processHardDrive(char *cPtr, newtComponent editPartition)
 	      memcpy(cFormat+37, szTemp, strlen(szTemp)); // File system
 	    }
 	  
-	  szDeviceName = new char[strlen(szFullDevice)+1]; // TODO: never deleted from memory
-	  strcpy(szDeviceName, szFullDevice);
+          szDeviceName = strdup(szFullDevice); // TOTO: never freed
 	  newtListboxAppendEntry(editPartition, cFormat, (void*)szDeviceName);
+          showDebug(9, "inserted[2]: %s\n", cFormat);
 	  //debugWin("add[%s] and *cPtr=%d and 1=%d and 2=%d and 3=%d and 4=%d", cFormat, *(cPtr), *(cPtr+1), *(cPtr+2), *(cPtr+3), *(cPtr+4));
 	}
     }
@@ -191,7 +193,7 @@ int fillPartitionList(newtComponent editPartition)
   char szTemp[1024];
   char *cPtr;
   int nLines;
-  int nSize;
+  unsigned int nSize;
   QWORD qwSize;
   int i;
   int nRes;
@@ -210,16 +212,28 @@ int fillPartitionList(newtComponent editPartition)
   nSize = 0;
   nLines = 0;
   memset(cBuffer, 0, sizeof(cBuffer));
-  while (!feof(fPart))
+  showDebug(9, "Reading /proc/partitions file\n");
+  while (nSize < sizeof(cBuffer) && !feof(fPart))
     {
       nRes = fgetc(fPart);
       if (nRes != -1)
-	cBuffer[nSize] = nRes;
+        cBuffer[nSize] = nRes;
+      else
+        showDebug(9, "error in fgetc\n");
       if (cBuffer[nSize] == '\n')
-	nLines++;		
-      nSize++;
+        {
+	  ++nLines;
+          showDebug(9, "%d lines read\n", nLines);
+        }
+      ++nSize;
     }
+  if (nSize < sizeof(cBuffer)) 
+    cBuffer[nSize] = '\0';
+  else
+    cBuffer[sizeof(cBuffer)-1] = '\0';
+
   fclose(fPart);
+  showDebug(9, "/proc/partitions read -> \n%s\n", cBuffer);
   
   cPtr = cBuffer;
   
@@ -241,9 +255,12 @@ int fillPartitionList(newtComponent editPartition)
       SNPRINTF(szFullDevice, "/dev/%s", szDevice);
       checkInodeForDevice(szFullDevice);
       nRes = detectFileSystem(szFullDevice, szTemp);
-      
+
+      showDebug(9, "device %s\n", szDevice);     
+ 
       if (nRes != -1) // if a removable media
 	{
+          showDebug(9, "removable media found: %s\n", szTemp);
 	  // add media
 	  memset(cFormat, ' ', 50);
 	  memset(cFormat+50, 0, 50);
@@ -256,9 +273,9 @@ int fillPartitionList(newtComponent editPartition)
 	  formatSize(qwSize, szTemp);	
 	  memcpy(cFormat+50, szTemp, strlen(szTemp)); // Size
 
-	  szDeviceName = new char[strlen(szFullDevice)+1]; // TODO: never deleted from memory
-	  strcpy(szDeviceName, szFullDevice);
+          szDeviceName = strdup(szFullDevice); // TODO: never freed
 	  newtListboxAppendEntry(editPartition, cFormat, (void*)szDeviceName);
+          showDebug(9, "inserted: %s\n", cFormat);
 
 	  // skip partitions
 	  cPtr = skipPartitionsEntries(cPtr);
@@ -320,9 +337,9 @@ int CSaveOptWindow::create(char *szImageFile, COptions options)
   m_labelSplit = newtLabel(1, 12, i18n("Image split mode"));
   m_radioSplitAuto = newtRadiobutton(1, 13, i18n("Automatic split (when no space left)"), !options.qwSplitSize, NULL);
   m_radioSplitSize = newtRadiobutton(1, 14, i18n("Into files whose size is:............"), !!options.qwSplitSize, m_radioSplitAuto);
-  SNPRINTF(szTemp, "%llu", (!!options.qwSplitSize) ? (options.qwSplitSize/1024) : 1440);
+  SNPRINTF(szTemp, "%llu", (!!options.qwSplitSize) ? (options.qwSplitSize/1024/1024) : 2048);
   m_editSplitSize = newtEntry(43, 14, szTemp, 8, NULL, 0);
-  m_labelSplitSizeKB = newtLabel(52, 14, i18n("Kilo-Bytes"));
+  m_labelSplitSizeKB = newtLabel(52, 14, i18n("MiB"));
   m_checkSplitWait = newtCheckbox(1, 15, i18n("Wait after each volume change"), (!!options.bSplitWait ? 'X' : ' '), " X", NULL);
 
   m_labelFinish = newtLabel(43, 7, i18n("If finished successfully:"));
@@ -330,13 +347,14 @@ int CSaveOptWindow::create(char *szImageFile, COptions options)
   m_radioFinishHalt = newtRadiobutton(43, 9, i18n("Halt"), options.dwFinish == FINISH_HALT, m_radioFinishWait);
   m_radioFinishReboot = newtRadiobutton(43, 10, i18n("Reboot"), options.dwFinish == FINISH_REBOOT, m_radioFinishHalt);
   m_radioFinishQuit = newtRadiobutton(43,11,i18n("Quit"), options.dwFinish == FINISH_QUIT, m_radioFinishReboot);
+  m_radioFinishLast = newtRadiobutton(43,12,i18n("Last"), options.dwFinish == FINISH_LAST, m_radioFinishQuit);
 
   addButtons();
   
   m_formMain = newtForm(NULL, NULL, 0);
   newtFormAddComponents(m_formMain, m_labelCompression, m_labelOptions, m_labelSplit, NULL);
   newtFormAddComponents(m_formMain, m_radioCompNone, m_radioCompGzip, m_radioCompBzip2, m_checkCheckBeforeSaving, m_checkAskDesc, m_checkOverwrite, NULL);
-  newtFormAddComponents(m_formMain, m_labelFinish, m_radioFinishWait, m_radioFinishHalt, m_radioFinishReboot, m_radioFinishQuit, NULL);	
+  newtFormAddComponents(m_formMain, m_labelFinish, m_radioFinishWait, m_radioFinishHalt, m_radioFinishReboot, m_radioFinishQuit, m_radioFinishLast, NULL);	
   newtFormAddComponents(m_formMain, m_radioSplitAuto, m_radioSplitSize, m_labelSplitSizeKB, m_editSplitSize, m_checkSplitWait, NULL);
   addHotKeys();
   
@@ -372,6 +390,8 @@ int CSaveOptWindow::getValues(COptions *options)
     options->dwFinish = FINISH_REBOOT;
   else if (newtRadioGetCurrent(m_radioFinishWait) == m_radioFinishQuit)
     options->dwFinish = FINISH_QUIT;
+  else if (newtRadioGetCurrent(m_radioFinishWait) == m_radioFinishLast)
+    options->dwFinish = FINISH_LAST;
   
   // get split mode
   if (newtRadioGetCurrent(m_radioSplitAuto) == m_radioSplitAuto)
@@ -380,11 +400,11 @@ int CSaveOptWindow::getValues(COptions *options)
     {	
       //options->dwSplitMode = SPLIT_SIZE;
 #ifdef HAVE_ATOLL
-      options->qwSplitSize = atoll(newtEntryGetValue(m_editSplitSize)) * 1024;
+      options->qwSplitSize = atoll(newtEntryGetValue(m_editSplitSize)) * 1024 * 1024;
 #else
   #ifdef HAVE_STRTOLL
       options->qwSplitSize = strtoll(newtEntryGetValue(m_editSplitSize), 
-         NULL, 10) * 1024;
+         NULL, 10) * 1024 * 1024;
   #else
     #error "no function to convert string to long long int"
   #endif // HAVE_STRTOLL
@@ -631,14 +651,17 @@ int CRestoreOptWindow::getValues(COptions *options)
 }
 
 // =======================================================
-int CRestoringWindow::create(char *szDevice, char *szImageFile, QWORD qwCurPartSize, DWORD dwCompressionMode, char *szOriginalDevice, char *szFileSystem, tm dateCreate, QWORD qwOrigPartSize)
+int CRestoringWindow::create(char *szDevice, char *szImageFile, QWORD qwCurPartSize, DWORD dwCompressionMode, char *szOriginalDevice, char *szFileSystem, tm dateCreate, QWORD qwOrigPartSize, COptions * options)
 {
   BEGIN;
   
   char szTemp[2048];
   char szTemp2[2048];
-  
-  SNPRINTF(szTemp, i18n("restore partition from image file"));
+ 
+  if (options->bSimulateMode)
+    SNPRINTF(szTemp, i18n("simulate partition restoration from image file"));
+  else
+    SNPRINTF(szTemp, i18n("restore partition from image file"));
   newtCenteredWindow(78, 20, szTemp);
   
   SNPRINTF(szTemp, i18n("Partition to restore:.............%s"), szDevice);
@@ -865,7 +888,7 @@ unsigned int CExceptionsGUI::windowAlreadyExist(char * img, char * path)
      " filename, please enter a full path\n(location + filename) "
      "without volume number at the end, \nand press \"Change\""), img);
 
-  nCont = windowError(i18n("Overwrite image ?"), szMess, i18n("Overwrite"), path);
+  nCont = windowError(i18n("Overwrite image?"), szMess, i18n("Overwrite"), path);
   RETURN_WORD(nCont);
 }
 
