@@ -26,6 +26,12 @@
   #include <sys/mount.h>
 #endif
 
+#include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/ioctl.h>
+#include <fcntl.h>
+
 #include "partimage.h"
 #include "imagefile.h"
 #include "common.h"
@@ -55,21 +61,31 @@ CImage::CImage(COptions * options)
   else
     {
 #ifdef MUST_LOGIN
-// we get login/pass from user
-      int nRes;
-      nRes = g_interface -> askLogin(login, passwd, 2048);
-      if (nRes)
+      if (strlen(options -> szUserName) && strlen(options -> szPassWord))
         {
-          free(login);
-          free(passwd);
-          THROW( ERR_CANCELED);
+          strncpy(login, options->szUserName, 2048);
+          strncpy(passwd, options->szPassWord, 2048);
+          *(login+2047) = '\0';
+          *(passwd+2047) = '\0';
         }
+      else 
+        {
+// we get login/pass from user
+          int nRes;
+          nRes = g_interface -> askLogin(login, passwd, 2048);
+          if (nRes)
+            {
+              free(login);
+              free(passwd);
+              THROW( ERR_CANCELED);
+            }
+        }	
 #else
 // fake login/passwd
       strcpy(login, "nologin");
       strcpy(passwd, "nopass");
 #endif
-          
+		
       m_cid = new CImageNet(options, options->bUseSSL);
       try { m_cid -> Connect(options, login, passwd, options->szServerName,
          options->dwServerPort); }
@@ -366,7 +382,7 @@ void CImage::read(char *cBuf, DWORD dwLength, bool bUpdateCRC)
   int nRes;
  
   if (dwLength != 1) // avoid having BUFFER_BLOCK_SIZE messages with imginfo and restmbr
-    showDebug(3, "begin of ci::read: %lu\n", dwLength);
+    showDebug(4, "begin of ci::read: size=%lu, pos=%lld\n", dwLength, qwTotalBytes);
 
   if (m_options.dwCompression == COMPRESS_NONE)
     nRes = fread(cBuf, 1, dwLength, m_fImageFile);
@@ -392,7 +408,6 @@ void CImage::read(char *cBuf, DWORD dwLength, bool bUpdateCRC)
       if (dwLength != 1)
 	showDebug(1, "READ ERROR: dwLength=%lu and dwRes=%lu and g_nThreadState=%d\n", dwLength, dwRes, g_nThreadState);
 
-      // if EOF
       if (g_nThreadState == THREAD_EOF)
 	{
 	  DWORD dwFirstRead = dwRes;
@@ -403,7 +418,7 @@ void CImage::read(char *cBuf, DWORD dwLength, bool bUpdateCRC)
 	      showDebug(1, "ci::read: exception raised from splitChangeImageFileRead()\n");
 	      throw excep;
 	    }
-
+          qwTotalBytes = 0LL;
 	  read(cBuf+dwFirstRead, dwLength-dwFirstRead, bUpdateCRC);
 	  m_guiRestore -> showImageFileInfo(get_szImageFilename(), m_options.dwCompression);
 	}
@@ -425,6 +440,8 @@ void CImage::read(char *cBuf, DWORD dwLength, bool bUpdateCRC)
 	m_guiRestore -> showImageFileInfo(get_szImageFilename(), m_options.dwCompression);
     }
 
+  if (dwLength != 1)
+    showDebug(4, "end of ci::read: %lu/%lu\n", dwRes, dwLength);
   RETURN;
 }
 
@@ -446,6 +463,16 @@ void CImage::splitChangeImageFileWrite()
   catch ( CExceptions * excep ) // catch errno
     { throw excep; } 
 
+  //execute /tmp/partimage-shell
+
+  if (m_options.bRunShell)
+    {
+      if (!fork()) // we're the child // no error check maid
+        execl("/tmp/partimage-shell", 
+           get_szImageFilename(), get_szPath(),
+           get_szOriginalFilename(), get_dwVolumeNumber());
+    }
+
   // wait for a keyboard keypress if need
   if (m_options.bSplitWait)
     {
@@ -535,8 +562,9 @@ void CImage::closeReading(bool bForceExit /*  = false */)
     nRes = fclose(m_fImageFile);	
   else if (m_options.dwCompression == COMPRESS_GZIP) // Gzip compression
     { 
+      char szTemp[2048];
       nRes = gzclose(m_gzImageFile);
-      //showDebug(3, "GZCLOSE: result=%s\n", formatGzipError(nRes, szTemp, sizeof(szTemp)));
+      showDebug(3, "GZCLOSE: result=%s\n", formatGzipError(nRes, szTemp, sizeof(szTemp)));
     }
   else if (m_options.dwCompression == COMPRESS_BZIP2) // Bzip2 compression
     BZ2_bzclose(m_bzImageFile);
@@ -627,18 +655,13 @@ int CImage::umountImageLocation()
 
   if ((m_options.szAutoMount) && (strlen(m_options.szAutoMount)) && (m_bIsMounted)) // if mounted
     {
-      debugWin("umount[%s]",m_szMountPoint);
+//      debugWin("umount[%s]",m_szMountPoint);
       showDebug(1, "umount[%s]",m_szMountPoint);
       nRes = m_cid->umountImageLocation(m_szMountPoint);
       if (nRes)
-	{
-	  THROW(ERR_AUTOUMOUNT, m_szMountDevice);
-	  //g_interface->msgBoxError(i18n("Cannot unmount %s"), m_szMountDevice);
-	  //RETURN_int(-1);	  
-	}
+        THROW(ERR_AUTOUMOUNT, m_szMountDevice);
       m_bIsMounted = false;
     }
-
   return 0;
 }
 
@@ -970,24 +993,32 @@ void CImage::openReading(CVolumeHeader *vh /* = NULL */)
   // ---- ask for another path while the file is not found
   while (m_cid->doesFileExists(m_cid->get_szImageFilename()) == false)
     {
-      SNPRINTF(szMess, i18n("Can't read the following volume file:\n%s\n%s"),
-	      get_szImageFilename(), i18n("Enter another full path (directory & name)"));
-      nRes = g_interface -> askText(szMess, i18n("Volume not found"),
-				    szAux, MAXPATHLEN);
-      if (nRes == -1)
-	THROW(ERR_CANCELED);
+      if ((m_options.szAutoMount) && (strlen(m_options.szAutoMount)))
+        { // Automount enabled -> wrong volume inserted, ask for the right
+          // one but don't change location
+	  nRes = g_interface -> msgBoxContinueCancel(
+             i18n("Wrong volume inserted"),
+	     i18n("You will be prompted to remove this media and insert the"
+             " right one"));
+          umountImageLocation();
+          mountImageLocation();
+	}
+      else
+	{
+          SNPRINTF(szMess,i18n("Can't read the following volume file:\n%s\n%s"),
+	          get_szImageFilename(),
+		  i18n("Enter another full path (directory & name)"));
+          nRes = g_interface -> askText(szMess, i18n("Volume not found"),
+                  szAux, MAXPATHLEN);
+          if (nRes == -1)
+    	    THROW(ERR_CANCELED);
       
-      set_szImageFilename(szAux);
-      extractFilepathFromFullPath(get_szImageFilename(), szAux);
-      set_szPath(szAux);
-      extractFilenameFromFullPath(get_szImageFilename(), szAux);
-      set_szOriginalFilename(szAux);
-      
-      /*
-	set_szImageFilename(aux);
-	extractFilepathFromFullPath(get_szImageFilename(), aux);
-	set_szPath(aux);
-      */
+          set_szImageFilename(szAux);
+          extractFilepathFromFullPath(get_szImageFilename(), szAux);
+          set_szPath(szAux);
+          extractFilenameFromFullPath(get_szImageFilename(), szAux);
+          set_szOriginalFilename(szAux);
+	}
     }
 
   nRes = pipe(nFd);
@@ -1101,9 +1132,11 @@ void CImage::openReading(CVolumeHeader *vh /* = NULL */)
     }
 
   // ----------- check image version
+  showDebug(4, "ImageFile version: [%s] and CURRENT_IMAGE_FORMAT=[%s]\n", headVolume.szVersion, CURRENT_IMAGE_FORMAT);
   if (strcmp(headVolume.szVersion, CURRENT_IMAGE_FORMAT) > 0)
     THROW(ERR_VERSION_NEWER, headVolume.szVersion);
-  else if (strcmp(headVolume.szVersion, CURRENT_IMAGE_FORMAT) < 0)
+  else if ((strcmp(headVolume.szVersion, CURRENT_IMAGE_FORMAT) < 0) && 
+	   (strcmp(headVolume.szVersion, "0.6.0") != 0))
     THROW(ERR_VERSION_OLDER, headVolume.szVersion);
 
   if (vh) // not the first volume -> image info
@@ -1131,7 +1164,7 @@ void CImage::openReading(CVolumeHeader *vh /* = NULL */)
     }
 
   RETURN;
-}	
+}
 
 // =======================================================
 int CImage::getCompressionLevelForImage(char *szFilename)
