@@ -45,14 +45,14 @@ void CInterface::showAboutDialog()
 {
   char szTitle[128];
   
-  SNPRINTF(szTitle, i18n("About Partition Image %s"), VERSION);
+  SNPRINTF(szTitle, i18n("About Partition Image %s"), PACKAGE_VERSION);
   
   msgBoxOk(szTitle, i18n("Partition Image is distributed under the GNU GPL 2"
      " (General Public License)\n\n"
      "version:...%s\n"
      "web:.......http://www.partimage.org/\n\n"
-     "Authors:\nFrancois Dupoux <fdupoux@partimage.org>\n"
-     "Franck Ladurelle <ladurelf@partimage.org>"), VERSION);
+     "Authors:\nFrancois Dupoux  <fdupoux@partimage.org>\n"
+     "Franck Ladurelle <ladurelf@partimage.org>"), PACKAGE_VERSION);
 }
 
 // =======================================================
@@ -272,6 +272,12 @@ void CInterface::ErrorReadingMBRMagic()
 
 
 
+// =======================================================
+WORD CInterface::ErrorBugBzip2()
+{
+  msgBoxCancel(i18n("Know bug"), i18n("Cannot restore the MBR from an image compressed with bzip2. You have to change the compression level."));
+  return 0;
+}
 
 // =======================================================
 WORD CInterface::ErrorLogAsRoot()
@@ -307,8 +313,22 @@ WORD CInterface::askRestore(char *szDevice, char *szImageFilename)
 // =======================================================
 WORD CInterface::askIgnoreFSError(char *szFsck)
 {
-  //return newtWinChoice(i18n("Warning"), i18n("Continue"), i18n("Cancel"), i18n("There are errors on the file system, or cannot run %s"));
-  return msgBoxContinueCancel(i18n("Warning"), i18n("There are errors on the file system, or cannot run %s"), szFsck);
+  return msgBoxContinueCancel(i18n("Warning"),
+     i18n("%s found errors on the file system."), szFsck);
+}
+
+// =======================================================
+WORD CInterface::askIgnoreNoFschk(char *szFsck)
+{
+  return msgBoxContinueCancel(i18n("Warning"),
+     i18n("Partimage will not check partition because %s does't exist."), szFsck);
+}
+
+// =======================================================
+WORD CInterface::askIgnoreDeniedFschk(char *szFsck)
+{
+  return msgBoxContinueCancel(i18n("Warning"),
+     i18n("Partimage will not check partition because of run access denied for %s."), szFsck);
 }
 
 // =======================================================
@@ -463,7 +483,7 @@ WORD CInterface::Error(CExceptions *excep, char *szFilename, char *szDevice/*=NU
       return ERR_QUIT;
 
     case ERR_PASSWD:
-      msgBoxError(i18n("incorrect password/login denied"));
+      msgBoxError(i18n("incorrect password or user not in partimagedusers file"));
       return ERR_QUIT;
 
     case ERR_TOOMANY:
@@ -491,10 +511,16 @@ WORD CInterface::Error(CExceptions *excep, char *szFilename, char *szDevice/*=NU
       return ERR_QUIT;
 
     case ERR_VOLUMENUMBER:
-      msgBoxError(i18n("This volume (%ld) is not the expected one: %ld"),
-		  dwArg2, dwArg1);
+      msgBoxError(i18n("This volume (%ld) is not the expected one: %ld."),
+		  dwArg1, dwArg2);
       return ERR_QUIT;
 
+    case ERR_HOST:
+	msgBoxError(i18n("Hostname/IP Error!\n"
+			 "Perhaps the hostname could not be resolved. "
+			 "(Try again using IP address)"));
+      return ERR_RETRY;
+
       /*
 	case ERR_SSL_CONNECT:
         msgBoxError(i18n("Problem with SSL. Ensure partimaged uses SSL"));
@@ -570,12 +596,12 @@ WORD CInterface::Error(CExceptions *excep, char *szFilename, char *szDevice/*=NU
       msgBoxError(i18n("Connexion refused by server: versions mismatch"));
       return ERR_QUIT;
 
-    case ERR_VERSION_OLDER:
+    case ERR_VERSION_NEWER:
       msgBoxError(i18n("The image file was created by a newer release (%s)" 
 		       " of Partition Image. Please update this software and retry."),
 		  szArg1);
       return ERR_QUIT;
-    case ERR_VERSION_NEWER:
+    case ERR_VERSION_OLDER:
       msgBoxError(i18n("The image file was created by an older release (%s)" 
 		       " of Partition Image. You must use the old version to restore"
 		       "your image or you have to recreate it with current version"),
@@ -608,6 +634,15 @@ WORD CInterface::Error(CExceptions *excep, char *szFilename, char *szDevice/*=NU
       msgBoxError(i18n("Cannot create temp file [%s]. No space left on device."), szArg1);
       return ERR_QUIT; 
 
+    case ERR_CREATESPACEFILEDENIED:
+#ifdef MUST_CHEUID
+      msgBoxError(i18n("Cannot create temp file in [%s]. Write access is denied"
+         " for user 'partimag'"), szArg1);
+#else
+      msgBoxError(i18n("Cannot create temp file in [%s]. Write access is denied."), szArg1);
+#endif
+      return ERR_QUIT; 
+
     case ERR_FILETOOLARGE:
       msgBoxError(i18n("File too large. (larger than 2 GB). Can't continue. This problem can come from the kernel, the glibc, the file system which where the image is written"));
       return ERR_QUIT;       
