@@ -104,9 +104,12 @@ void CInterfaceNewt::msgBoxOk(char *szTitle, char *szText, ...)
 {
   va_list args;
   
-  va_start(args, szText);
-  newtWinMessagev(szTitle, i18n("Ok"), szText, args);
-  va_end(args);
+  if (!getBatchMode())
+    {
+      va_start(args, szText);
+      newtWinMessagev(szTitle, i18n("Ok"), szText, args);
+      va_end(args);
+    }
 }
 
 // =======================================================
@@ -114,9 +117,12 @@ void CInterfaceNewt::msgBoxCancel(char *szTitle, char *szText, ...)
 {
   va_list args;
   
-  va_start(args, szText);
-  newtWinMessagev(szTitle, i18n("Cancel"), szText, args);
-  va_end(args);
+  if (!getBatchMode())
+    {
+      va_start(args, szText);
+      newtWinMessagev(szTitle, i18n("Cancel"), szText, args);
+      va_end(args);
+    }
 }
 
 // =======================================================
@@ -135,13 +141,21 @@ int CInterfaceNewt::msgBoxContinueCancel(char *szTitle, char *szText, ...)
   char szBuf[4096];
   va_list args;
   int nRes;
-  
+
   va_start(args, szText);
   vsnprintf(szBuf, sizeof(szBuf), szText, args);
-  nRes = newtWinChoice(szTitle, i18n("Continue"), i18n("Cancel"), szBuf);
   va_end(args);
-  
-  return ((nRes == 1) ? MSGBOX_CONTINUE : MSGBOX_CANCEL);
+ 
+  if (!getBatchMode())
+    {
+      nRes = newtWinChoice(szTitle, i18n("Continue"), i18n("Cancel"), szBuf);
+      return ((nRes == 1) ? MSGBOX_CONTINUE : MSGBOX_CANCEL);
+    }
+  else
+    {
+      showDebug(1, "Error: %s\n", szBuf);
+      return MSGBOX_CANCEL;
+    }
 }
 
 // =======================================================
@@ -150,14 +164,21 @@ int CInterfaceNewt::msgBoxYesNo(char *szTitle, char *szText, ...)
   char szBuf[4096];
   va_list args;
   int nRes;
-  
+
   va_start(args, szText);
   vsnprintf(szBuf, sizeof(szBuf), szText, args);
-  nRes = newtWinChoice(szTitle, i18n("Yes"), i18n("No"), szBuf);
-  //newtWinMessagev(szTitle, i18n("Ok"), szText, args);
   va_end(args);
-  
-  return ((nRes == 1) ? MSGBOX_YES : MSGBOX_NO);
+
+  if (!getBatchMode())
+    {
+      nRes = newtWinChoice(szTitle, i18n("Yes"), i18n("No"), szBuf);
+      return ((nRes == 1) ? MSGBOX_YES : MSGBOX_NO);
+    }
+  else
+    {
+      showDebug(1, "Error: %s\n", szBuf);
+      return MSGBOX_NO;
+    }
 }
 
 // =======================================================
@@ -166,7 +187,8 @@ void CInterfaceNewt::StatusLine(char *str)
   char szBuf[1024];
   if (m_bPushDone)
     newtPopHelpLine(); // work only if there is a previous push
-  snprintf(szBuf, sizeof(szBuf), "%s [* to cancel, CtrlS, CtrlQ]", str);
+  snprintf(szBuf, sizeof(szBuf),
+     "%s [* to cancel, CtrlS to pause, CtrlQ to resume]", str);
   newtPushHelpLine(szBuf); // str
   m_bPushDone = true;
 }
@@ -185,7 +207,7 @@ int CInterfaceNewt::guiInitMainWindow(char *szDevice, char *szImageFile, char *s
 
   showDebug(8, "guiInitMainWindow\n");
   
-  SNPRINTF(szTemp, "Partition Image %s", VERSION);
+  SNPRINTF(szTemp, "Partition Image %s", PACKAGE_VERSION);
   newtCenteredWindow(67, 20, szTemp);
   
   labelPartition = newtLabel(1, 0, i18n("* Partition to save/restore"));
@@ -201,7 +223,11 @@ int CInterfaceNewt::guiInitMainWindow(char *szDevice, char *szImageFile, char *s
 #ifdef HAVE_SSL
   checkSsl = newtCheckbox(5, 19, i18n("Encrypt data on the network with SSL"), (*bSsl ? 'X' : ' '), " X", NULL);
 #else
-  checkSsl = newtLabel(5, 19,i18n("SSL disabled at compile time"));
+  #ifdef MUST_LOGIN
+    checkSsl = newtLabel(5, 19,i18n("SSL disabled at compile time"));
+  #else
+    checkSsl = newtLabel(5, 19,i18n("SSL&login disabled at compile time"));
+  #endif
 #endif
   labelNetwork = newtLabel(5, 18, i18n("IP/name of the server:"));
   editNetwork = newtEntry(28, 18, szNetworkIP, 25, NULL, 0);
@@ -434,7 +460,7 @@ int CInterfaceNewt::askLogin(char *szLogin, char *szPasswd, WORD size)
 }
 
 // =======================================================
-int CInterfaceNewt::askNewPath(char * szOrigFilename, DWORD dwVolume, 
+WORD CInterfaceNewt::askNewPath(char * szOrigFilename, DWORD dwVolume, 
    char * szPath, char * szNewPath, WORD size)
 {
   char szMess[2048];
