@@ -5,7 +5,7 @@
     copyright            : (C) 2000 by Franois Dupoux
     email                : fdupoux@partimage.org
  ***************************************************************************/
-// $Revision: 1.54 $
+// $Revision: 1.70 $
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -37,6 +37,8 @@
 #include <sys/stat.h>
 #include <sys/file.h>
 #include <signal.h>
+#include <sys/mount.h>
+#include <errno.h>
 
 #include "fs_ext2.h"
 #include "fs_fat.h"
@@ -57,6 +59,7 @@
 #include "buffer.h"
 #include "exceptions.h"
 #include "interface_newt.h"
+#include "interface_none.h"
 #include "mbr_backup.h"
 
 extern char *optarg;
@@ -71,35 +74,39 @@ time_t g_timeEnd=0;   // End of the operation
 #ifdef HAVE_GETOPT_H
 static struct option const long_options[] =
 {
-  {"compress", required_argument, NULL, 'z'},
-  {"overwrite", no_argument, NULL, 'o'},
-  {"volume", required_argument, NULL, 'V'},
-  {"waitvol", no_argument, NULL, 'w'},
-  {"erase", no_argument, NULL, 'e'},
-  {"nocheck", no_argument, NULL, 'c'},
-  {"allowmnt", no_argument, NULL, 'm'},
-  {"nodesc", no_argument, NULL, 'd'},
-  {"help", no_argument, NULL, 'h'},
-  {"version", no_argument, NULL, 'v'},
-  {"compilinfo", no_argument, NULL, 'i'},
-  {"batch", no_argument, NULL, 'b'},
-  {"finish", required_argument, NULL, 'f'},
-  {"server", required_argument, NULL, 's'},
-  {"port", required_argument, NULL, 'p'},
   {"automnt", required_argument, NULL, 'a'},
+  {"batch", no_argument, NULL, 'b'},
+  {"fully-batch", required_argument, NULL, 'B'},
+  {"nocheck", no_argument, NULL, 'c'},
+  {"nodesc", no_argument, NULL, 'd'},
+  {"erase", no_argument, NULL, 'e'},
+  {"finish", required_argument, NULL, 'f'},
   {"debug", required_argument, NULL, 'g'},
-  {"nosync", no_argument, NULL, 'y'},
-  {"nossl", no_argument, NULL, 'n'},
+  {"help", no_argument, NULL, 'h'},
+  {"compilinfo", no_argument, NULL, 'i'},
   {"nombr", no_argument, NULL, 'M'},
+  {"allowmnt", no_argument, NULL, 'm'},
+  {"nossl", no_argument, NULL, 'n'},
+  {"overwrite", no_argument, NULL, 'o'},
+  {"password", required_argument, NULL, 'P'},
+  {"port", required_argument, NULL, 'p'},
   {"simulate", no_argument, NULL, 'S'},
+  {"server", required_argument, NULL, 's'},
+  {"username", required_argument, NULL, 'U'},
+  {"volume", required_argument, NULL, 'V'},
+  {"version", no_argument, NULL, 'v'},
+  {"waitvol", no_argument, NULL, 'w'},
+  {"runshell", no_argument, NULL, 'X'},
+  {"nosync", no_argument, NULL, 'y'},
+  {"compress", required_argument, NULL, 'z'},
   {NULL, 0, NULL, 0}
 };
 #endif
 
-static char optstring[]="z:oV:ecmdhf:s:p:bwg:vynSMa:i";
-FILE *g_fDebug; // debug file
-FILE *g_fLocalDebug; // debug file
-CInterfaceNewt* g_interface;
+static char optstring[]="z:oV:ecmdhf:s:p:bwg:vynSMa:iU:P:XB:";
+FILE * g_fDebug; // debug file
+FILE * g_fLocalDebug; // debug file
+CInterface * g_interface;
 bool g_bSigInt = false;
 bool g_bSigKill = false;
 QWORD g_qwCopiedBytesCount=0;
@@ -164,46 +171,62 @@ int main(int argc, char *argv[])
   options.dwServerPort = OPTIONS_DEFAULT_SERVERPORT;
   options.bBatchMode = OPTIONS_DEFAULT_AUTOSTART;
   options.bSplitWait = OPTIONS_DEFAULT_SPLIT_WAIT;
+  options.bRunShell = OPTIONS_DEFAULT_RUN_SHELL;
   options.bSync = OPTIONS_DEFAULT_SYNC;
   options.dwDebugLevel = DEFAULT_DEBUG_LEVEL; // defined by configure.in
   strncpy(options.szServerName, OPTIONS_DEFAULT_SERVERNAME, MAX_HOSTNAMESIZE);
   strncpy(options.szAutoMount, OPTIONS_DEFAULT_AUTOMOUNT, MAXPATHLEN);
+  strncpy(options.szUserName, OPTIONS_DEFAULT_USERNAME, MAX_USERNAMELEN);
+  strncpy(options.szPassWord, OPTIONS_DEFAULT_PASSWORD, MAX_PASSWORDLEN);
+  *(options.szFullyBatchMode) = 0;
   
   // initialize language for i18n
   setlocale(LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
-  
-  /*if (g_wEndian == ENDIAN_LITTLE)
-    {
-    if (checkStructSizes() == -1)
-    return EXIT_FAILURE;
-    }*/
-  checkStructSizes();
+
+  // Very important: do not remove (else, uncompatible image will be produced)
+  if (checkStructSizes() == -1)
+     return EXIT_FAILURE;
 
   // read command line
 #ifdef HAVE_GETOPT_H
-  while ((nOptCh = getopt_long (argc, argv, optstring, long_options, NULL))
-	 != EOF)
+  while ((nOptCh = getopt_long (argc, argv, optstring, long_options, NULL)) != EOF)
 #else
     while ((nOptCh = getopt (argc, argv, optstring)) != -1)
 #endif
-
       {
 	switch(nOptCh)
 	  {
 	  case 's': // partimaged server's ip addr
 	    strncpy(options.szServerName, optarg, MAX_HOSTNAMESIZE);
+            *(options.szServerName+MAX_HOSTNAMESIZE-1) = '\0';
 	    break;
 	  case 'a': // auto mount options
 	    strncpy(options.szAutoMount, optarg, MAXPATHLEN);
+            *(options.szAutoMount+MAXPATHLEN-1) = '\0';
 	    break;
+	  case 'U': // username for serverauth
+            strncpy(options.szUserName, optarg, MAX_USERNAMELEN);
+            *(options.szUserName+MAX_USERNAMELEN-1) = '\0';
+            break;
+	  case 'B': // fully batchmode: don't do any newt or anything
+	    strncpy(options.szFullyBatchMode, optarg, 2047);
+	    options.bBatchMode = true;
+	    break;
+	  case 'P': // password for serverauth
+            strncpy(options.szPassWord, optarg, MAX_PASSWORDLEN);
+            *(options.szPassWord+MAX_PASSWORDLEN-1) = '\0';
+            break;
 	  case 'b': // batchmode: don't wait for user action (return)
 	    options.bBatchMode = true;
 	    break;
 	  case 'w': // wait after each new colume
 	    options.bSplitWait = true;
 	    break;
+          case 'X': // run shell after all volume
+            options.bRunShell = true;
+            break; 
 	  case 'M': // do not create a backup of the MBR
 	    options.bBackupMBR = false;
 	    break;
@@ -251,8 +274,8 @@ int main(int argc, char *argv[])
 	  
 	  case 'V': // split image into multiple files
 	  
-	    options.qwSplitSize = atol(optarg) * 1024;
-	    fprintf (stderr, i18n("Volume size: %llu bytes (%ld KB)\n"),
+	    options.qwSplitSize = atol(optarg) * 1024 * 1024;
+	    fprintf (stderr, i18n("Volume size: %llu bytes (%ld MiB)\n"),
 		     options.qwSplitSize, atol(optarg));
 	    break;
 	  
@@ -283,7 +306,7 @@ int main(int argc, char *argv[])
 	  
 	  case 'v': // version
 	    printf(i18n("Partition Image version %s (distributed under the"
-			" GNU GPL2).\n"), VERSION);
+			" GNU GPL2).\n"), PACKAGE_VERSION);
 	    return EXIT_SUCCESS;
 	    break;
 
@@ -302,16 +325,6 @@ int main(int argc, char *argv[])
 	  }
       }
 
-  // local debug
-#ifdef PARTIMAGE_LOCAL_DEBUG
-  g_fLocalDebug = openFileDescriptorSecure(PARTIMAGE_LOCAL_DEBUG, "a", /*O_RDONLY | O_LARGEFILE | */  O_WRONLY | O_CREAT | O_NOFOLLOW | O_TRUNC, S_IRUSR | S_IWUSR);
-  if (!g_fLocalDebug)
-    g_fLocalDebug = stderr;
-  else
-    fchmod(fileno(g_fLocalDebug), S_IRUSR|S_IWUSR);
-  //g_fLocalDebug = fopen(PARTIMAGE_LOCAL_DEBUG, "w+");
-#endif // PARTIMAGE_LOCAL_DEBUG
-
   // debug file
   g_fDebug = NULL;
   if (options.dwDebugLevel)
@@ -344,7 +357,7 @@ int main(int argc, char *argv[])
   else
     {
       showDebug(1, "%s: Partition Image version %s (DebugLevel %lu used, "
-		"MainThread=%d)\n",PARTIMAGE_LOG, VERSION, options.dwDebugLevel, 
+		"MainThread=%d)\n",PARTIMAGE_LOG, PACKAGE_VERSION, options.dwDebugLevel, 
 		getpid());
       showDebug(1, "========================================================="
 		"==============================\n\n\n");
@@ -412,6 +425,7 @@ int main(int argc, char *argv[])
   // init CRC table
   initCrcTable(g_dwCrcTable);
 
+  if (!options.szFullyBatchMode || !strlen(options.szFullyBatchMode)) {
   // signal
   /*struct sigaction saOld, saNew;
     saNew.sa_handler = catch_sigint;
@@ -421,7 +435,8 @@ int main(int argc, char *argv[])
     sigaction(SIGINT, &saNew, &saOld);*/
   signal(SIGTERM, catch_sigint);
   signal(SIGINT, catch_sigint);
-  signal(SIGSEGV, catch_sigint); // segmentation fault
+//  signal(SIGSEGV, catch_sigint); // segmentation fault
+  }
    
   // endianess
   switch(g_wEndian)
@@ -441,7 +456,11 @@ int main(int argc, char *argv[])
 
   // init interface
   showDebug(8, "initialize interface\n");
-  g_interface = new PARTIMAGE_INTERFACE(options.bBatchMode);
+  if (!options.szFullyBatchMode || !strlen(options.szFullyBatchMode))
+    g_interface = new PARTIMAGE_INTERFACE(options.bBatchMode);
+  else
+    g_interface = new CInterfaceNone(options.szFullyBatchMode);
+  
   showDebug(8, "interface ok\n");
 
   // ----------- check the user is logged as root
@@ -504,6 +523,16 @@ int main(int argc, char *argv[])
     }
   showDebug(8, "ok for options\n");
 
+  if ((isDevfsEnabled()) && (!isDevfsMounted()))
+  {
+    g_interface->msgBoxCancel(i18n("Warning"),
+       i18n("You have devfs enabled but not mounted. You have to do it to "
+            "continue. You can also boot with non-devfs kernel if you have "
+	    "one."));
+    delete g_interface;
+    return EXIT_FAILURE;
+  }
+  
  beginMainWin:
 
   // if no command line or invalid action
@@ -551,7 +580,8 @@ int main(int argc, char *argv[])
       else
 	{
 	  showDebug(1, "The file system of [%s] is [%s], and is not supported\n", szDevice, szFileSystem);
-	  g_interface->msgBoxError("The file system of [%s] is [%s], and is not supported", szDevice, szFileSystem);
+	  if (!options.bBatchMode)
+	    g_interface->msgBoxError("The file system of [%s] is [%s], and is not supported", szDevice, szFileSystem);
 	  nRes = -1;
 	}
 //#endif // DEVEL_SUPPORT
@@ -617,7 +647,10 @@ int main(int argc, char *argv[])
       
     default: // exit
       if (options.bSync)
-	sync();
+        {
+          g_interface -> StatusLine(i18n("commiting buffer cache to disk."));
+	  sync();
+        }
       closeDebugFiles();
       delete g_interface; 
       return EXIT_SUCCESS;
@@ -627,7 +660,10 @@ int main(int argc, char *argv[])
   showDebug(8, "sync\n"); 
   // update the disk IO
   if (options.bSync)
-    sync();
+    {
+      g_interface -> StatusLine(i18n("commiting buffer cache to disk."));
+      sync();
+    }
   
   // ---- statistics about time/CPU infos ----
   rusage rusageSelf, rusageChild;
@@ -666,15 +702,18 @@ int main(int argc, char *argv[])
     {
       showDebug(1, "End of operation: FAILED\n");
       closeDebugFiles();
+      delete g_interface; 
+      return EXIT_FAILURE;
     }
   else // SUCCESS
     {	
       showDebug(1, "End of operation: SUCCESS\n");
-      closeDebugFiles();
-      pthread_mutex_destroy(&g_mutexDebug);
+//      closeDebugFiles();
+//      pthread_mutex_destroy(&g_mutexDebug);
       
       if (options.dwFinish == FINISH_WAIT)
 	{
+	  showDebug(1, "option FINISH_WAIT\n");
 	  if (nChoice == OPERATION_RESTMBR)
 	    {
 	      SNPRINTF(szTemp, i18n("The MBR was successfully restored"));
@@ -689,37 +728,103 @@ int main(int argc, char *argv[])
    
 	  if (nChoice != OPERATION_IMGINFO)
 	    g_interface->msgBoxOk(i18n("Success"), szTemp);
-	}
-      else if (options.dwFinish == FINISH_HALT)
-	{
-	  nRes = system("/sbin/shutdown -h now");
-	  if (nRes != 0)
-	    {
-	      fprintf(stderr, i18n("Error: Cannot halt the computer"));
-	      delete g_interface; 
-	      return EXIT_SUCCESS;
-	    }
-	}
-      else if (options.dwFinish == FINISH_REBOOT)
-	{
-	  nRes = system("/sbin/shutdown -r now");
-	  if (nRes != 0)
-	    {
-	      fprintf(stderr, i18n("Error: Cannot reboot the computer"));
-	      delete g_interface; 
-	      return EXIT_SUCCESS;
-	    }
-	}
-      else if (options.dwFinish == FINISH_QUIT)
-	{
 	  delete g_interface; 
 	  return EXIT_SUCCESS;
 	}
-      
+      else if (options.dwFinish == FINISH_HALT)
+	{
+	  showDebug(1, "option FINISH_HALT\n");
+	  delete g_interface; 
+	  nRes = system("/sbin/shutdown -h now");
+          nRes = system("/sbin/poweroff");
+// if we reach this point, it's because shutdown failed
+	  fprintf(stderr, i18n("Error: Cannot halt the computer"));
+	  return EXIT_SUCCESS;
+	}
+      else if (options.dwFinish == FINISH_REBOOT)
+	{
+	  showDebug(1, "option FINISH_REBOOT\n");
+	  delete g_interface; 
+	  nRes = system("/sbin/shutdown -r now");
+          nRes = system("/sbin/reboot");
+// if we reach this point, it's because shutdown failed
+	  fprintf(stderr, i18n("Error: Cannot reboot the computer"));
+	  return EXIT_SUCCESS;
+	}
+      else if (options.dwFinish == FINISH_QUIT)
+	{
+	  showDebug(1, "option FINISH_QUIT\n");
+	  delete g_interface; 
+	  return EXIT_SUCCESS;
+	}
+      else if (options.dwFinish == FINISH_LAST)
+	{
+	  showDebug(1, "option FINISH_LAST\n");
+	  int nLockFile;
+	  FILE * nCountFile;
+	  char szValue[11];
+	  int nValue, nRetries;
+	  ssize_t nRes;
+
+	  nRetries = 5;
+          do {
+	    nLockFile = open(FINISH_LAST_COUNTFILE_LOCK, O_CREAT|O_EXCL,
+               O_RDONLY);
+	    if (nLockFile == -1)
+	      {
+                --nRetries;
+		sleep(2);
+	      }
+	  } while (nLockFile == -1 && nRetries);
+	  if (!nRetries)
+	    {
+    	      delete g_interface; 
+	      fprintf(stderr, i18n("Error: Cannot count remaining partimages"));
+	      return EXIT_SUCCESS;
+	    }
+          nCountFile = fopen(FINISH_LAST_COUNTFILE, "r");
+	  if (nCountFile == NULL)
+	    {
+    	      delete g_interface; 
+	      fprintf(stderr, i18n("Error: Cannot count remaining partimages"));
+	      return EXIT_SUCCESS;
+	    }
+	  fscanf(nCountFile, "%d\n", &nValue);
+	  showDebug(1, "RESTANT: %d\n", nValue);
+	  if (nValue <= 1)
+	    { // we are the last running partimage -> shutdown
+	      delete g_interface; 
+	      fclose(nCountFile);
+	      close(nLockFile);
+	      unlink(FINISH_LAST_COUNTFILE_LOCK);
+	      unlink(FINISH_LAST_COUNTFILE);
+//	      nRes = system("/sbin/shutdown -r now");
+//              nRes = system("/sbin/reboot");
+// if we reach this point, it's because shutdown failed
+	      fprintf(stderr, i18n("DEBUG: reboot the computer"));
+	      return EXIT_SUCCESS;
+	    }
+	  else
+	    {
+              --nValue;
+	      fclose(nCountFile);
+              nCountFile = fopen(FINISH_LAST_COUNTFILE, "w");
+	      if (nCountFile == NULL)
+	        {
+    	          delete g_interface; 
+	          fprintf(stderr, i18n("Error: Cannot count remaining "
+                     "partimages"));
+	          return EXIT_SUCCESS;
+	        }
+	      fprintf(nCountFile, "%d\n", nValue);
+	      fclose(nCountFile);
+	      close(nLockFile);
+	      unlink(FINISH_LAST_COUNTFILE_LOCK);
+    	      delete g_interface; 
+	      return EXIT_SUCCESS;
+	    }
+        }
     }
-  
-  delete g_interface; 
-  return EXIT_SUCCESS;
 }
 
 // =======================================================
@@ -729,10 +834,6 @@ void closeDebugFiles()
     fclose(g_fDebug);
 
   // local debug
-#ifdef PARTIMAGE_LOCAL_DEBUG
-  if (g_fLocalDebug)
-    fclose(g_fLocalDebug);
-#endif // PARTIMAGE_LOCAL_DEBUG
 }
 
 // =======================================================
@@ -787,139 +888,148 @@ void usage()
 	      "* -g,  --debug=X      set the debug level to X (default: 1):\n"
 	      "* -n,  --nossl        disable SSL in network mode\n"
 	      "* -S,  --simulate     simulation of restoration mode\n"
-	      "* -aX, --automnt=X    Automatic mount with X options. Read the doc for more details\n"),
-	 VERSION, isVersionStable(VERSION) ? i18n("stable") : i18n("unstable")); 
+	      "* -aX, --automnt=X    Automatic mount with X options. Read the doc for more details\n"
+		  "* -UX  --username=X   Username to authenticate to server\n"
+		  "* -PX  --Password=X   Password for authentication of user to server\n"),
+	 PACKAGE_VERSION, isVersionStable(PACKAGE_VERSION) ? i18n("stable") : i18n("unstable")); 
   printf("===============================================================================\n");
 }
 
 // =======================================================
 int checkStructSizes()
 {
-  // ---- check types sizes
+   // ---- check types sizes
 
-  if (sizeof(unsigned long int) != 4)
-    {
+   if (sizeof(unsigned long int) != 4)
+   {
       fprintf (stderr, "Error: sizeof(DWORD) != 4 (%d)\n", sizeof(unsigned long int));
-      RETURN_int(-1);
-    }
+      goto errcheck;
+   }
 
-  if (sizeof(unsigned long long int) != 8)
-    {
+   if (sizeof(unsigned long long int) != 8)
+   {
       fprintf (stderr, "Error: sizeof(QWORD) != 8 (%d)\n", sizeof(unsigned long long int));
-      RETURN_int(-1);
-    }
+      goto errcheck;
+   }
 
-  // ---- check struct sizes
+   // ---- check struct sizes
 
-  // check main header size is 16384
-  if (sizeof(CMainHeader) != 16384)
-    {
+   // check main header size is 16384
+   if (sizeof(CMainHeader) != 16384)
+   {
       fprintf (stderr, "Error: main header size != 16384 (%d)\n",
-         sizeof(CMainHeader));
-      RETURN_int(-1);
-    }
+	       sizeof(CMainHeader));
+      goto errcheck;
+   }
   
-  // check ext2 header size is 16384
-  if (sizeof(CInfoExt2Header) != INFOS_STRUCT_SIZE)
-    {	
+   // check ext2 header size is 16384
+   if (sizeof(CInfoExt2Header) != INFOS_STRUCT_SIZE)
+   {	
       fprintf (stderr, "Error: ext2 header size != %d (%d)\n", INFOS_STRUCT_SIZE,
-         sizeof(CInfoExt2Header));
-      RETURN_int(-1);
-    }
+	       sizeof(CInfoExt2Header));
+      goto errcheck;
+   }
   
-  // check reiserfs header size is 16384
-  if (sizeof(CInfoReiserHeader) != INFOS_STRUCT_SIZE)
-    {	
+   // check reiserfs header size is 16384
+   if (sizeof(CInfoReiserHeader) != INFOS_STRUCT_SIZE)
+   {	
       fprintf (stderr, "Error: reiserfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
-         sizeof(CInfoReiserHeader));
-      RETURN_int(-1);
-    }
+	       sizeof(CInfoReiserHeader));
+      goto errcheck;
+   }
   
-  // check fat header size is 16384
-  if (sizeof(CInfoFatHeader) != INFOS_STRUCT_SIZE)
-    {	
+   // check fat header size is 16384
+   if (sizeof(CInfoFatHeader) != INFOS_STRUCT_SIZE)
+   {	
       fprintf (stderr, "Error: fat header size != %d (%d)\n", INFOS_STRUCT_SIZE,
-         sizeof(CInfoFatHeader));
-      RETURN_int(-1);
-    }
+	       sizeof(CInfoFatHeader));
+      goto errcheck;
+   }
   
-  // check ntfs header size is 16384
-  if (sizeof(CInfoNtfsHeader) != INFOS_STRUCT_SIZE)
-    {	
+   // check ntfs header size is 16384
+   if (sizeof(CInfoNtfsHeader) != INFOS_STRUCT_SIZE)
+   {	
       fprintf (stderr, "Error: ntfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
-         sizeof(CInfoNtfsHeader));
-      RETURN_int(-1);
-    }
+	       sizeof(CInfoNtfsHeader));
+      goto errcheck;
+   }
   
-  // check hpfs header size is 16384
-  if (sizeof(CInfoHpfsHeader) != INFOS_STRUCT_SIZE)
-    {	
+   // check hpfs header size is 16384
+   if (sizeof(CInfoHpfsHeader) != INFOS_STRUCT_SIZE)
+   {	
       fprintf (stderr, "Error: hpfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
-         sizeof(CInfoHpfsHeader));
-      RETURN_int(-1);
-    }
+	       sizeof(CInfoHpfsHeader));
+      goto errcheck;
+   }
   
-  // check jfs header size is 16384
-  if (sizeof(CInfoJfsHeader) != INFOS_STRUCT_SIZE)
-    {	
+   // check jfs header size is 16384
+   if (sizeof(CInfoJfsHeader) != INFOS_STRUCT_SIZE)
+   {	
       fprintf (stderr, "Error: jfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
-         sizeof(CInfoJfsHeader));
-      RETURN_int(-1);
-    }
+	       sizeof(CInfoJfsHeader));
+      goto errcheck;
+   }
   
-  // check xfs header size is 16384
-  if (sizeof(CInfoXfsHeader) != INFOS_STRUCT_SIZE)
-    {	
+   // check xfs header size is 16384
+   if (sizeof(CInfoXfsHeader) != INFOS_STRUCT_SIZE)
+   {	
       fprintf (stderr, "Error: xfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
-         sizeof(CInfoXfsHeader));
-      RETURN_int(-1);
-    }
+	       sizeof(CInfoXfsHeader));
+      goto errcheck;
+   }
  
-  // check hfs header size is 16384
-  if (sizeof(CInfoHfsHeader) != INFOS_STRUCT_SIZE)
-    {	
+   // check hfs header size is 16384
+   if (sizeof(CInfoHfsHeader) != INFOS_STRUCT_SIZE)
+   {	
       fprintf (stderr, "Error: hfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
-         sizeof(CInfoHfsHeader));
-      RETURN_int(-1);
-    }
+	       sizeof(CInfoHfsHeader));
+      goto errcheck;
+   }
  
-  // check ufs header size is 16384
-  if (sizeof(CInfoUfsHeader) != INFOS_STRUCT_SIZE)
-    {	
+   // check ufs header size is 16384
+   if (sizeof(CInfoUfsHeader) != INFOS_STRUCT_SIZE)
+   {	
       fprintf (stderr, "Error: jfs header size != %d (%d)\n", INFOS_STRUCT_SIZE,
-         sizeof(CInfoUfsHeader));
-      RETURN_int(-1);
-    }
+	       sizeof(CInfoUfsHeader));
+      goto errcheck;
+   }
  
-  // check tail size is 16384
-  if (sizeof(CMainTail) != 16384)
-    {	
+   // check tail size is 16384
+   if (sizeof(CMainTail) != 16384)
+   {	
       fprintf (stderr, "Error: main footer size != 16384 (%d)\n", sizeof(CMainTail));
-      RETURN_int(-1);
-    }
+      goto errcheck;
+   }
   
-  // check volume header size is 512
-  if (sizeof(CVolumeHeader) != 512)
-    {	
+   // check volume header size is 512
+   if (sizeof(CVolumeHeader) != 512)
+   {	
       fprintf (stderr, "Error: volume hedaer size != 512 (%d)\n", sizeof(CVolumeHeader));
-      RETURN_int(-1);
-    }
+      goto errcheck;
+   }
  
-  // check MBR size is 2048
-  if (sizeof(CMbr) != 2048)
-    {	
+   // check MBR size is 2048
+   if (sizeof(CMbr) != 2048)
+   {	
       fprintf (stderr, "Error: MBR size != 2048 (%d)\n", sizeof(CMbr));
-      RETURN_int(-1);
-    }
+      goto errcheck;
+   }
  
-  // check volume header size is 512
-  if (sizeof(CLocalHeader) != 16384)
-    {	
+   // check volume header size is 512
+   if (sizeof(CLocalHeader) != 16384)
+   {	
       fprintf (stderr, "Error: local header size != 16384 (%d)\n", sizeof(CLocalHeader));
-      RETURN_int(-1);
-    }
+      goto errcheck;
+   }
 
-  return 0;
+   return 0; // success
+
+  errcheck:
+   printf ("This version has been compiled with an uncompatible version of gcc.\n");
+   //printf("DetailSize: sizeof(DWORD)=%d\n", sizeof(DWORD));
+   //printf("DetailSize: sizeof(QWORD)=%d\n", sizeof(QWORD));
+   //printf("DetailSize: sizeof(struct tm)=%d\n", sizeof(struct tm));
+   RETURN_int(-1); // error
 }
 
 // =======================================================
