@@ -5,7 +5,7 @@
     copyright            : (C) 2000 by Franois Dupoux
     email                : fdupoux@partimage.org
  ***************************************************************************/
-// $Revision: 1.13 $
+// $Revision: 1.16 $
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -50,7 +50,7 @@ bool isDriveReady(char *szDevice)
       RETURN_bool(false);
     }
 
-  nRes = fread(cTempBuf, min(sizeof(cTempBuf), 512), 1, fTest);
+  nRes = fread(cTempBuf, my_min(sizeof(cTempBuf), 512), 1, fTest);
   fclose(fTest);
 
   if (nRes == 1) // if possible to read data from device
@@ -184,7 +184,7 @@ int mbrCopyFileToString(char *cDest, char *szFilepath, int nMaxSize)
 
   FILE *fInfo;
   int i;
-  char c;
+  int c;
 
   // ---- open file
   fInfo = fopen(szFilepath, "rb");
@@ -202,7 +202,7 @@ int mbrCopyFileToString(char *cDest, char *szFilepath, int nMaxSize)
   // ---- close file
   fclose(fInfo);
 
-  RETURN_int(0);
+  return(0);
 }
 
 // =============================================================================
