@@ -5,7 +5,7 @@
     copyright            : (C) 2000 by Franois Dupoux
     email                : fdupoux@partimage.org
  ***************************************************************************/
-// $Revision: 1.77 $
+// $Revision: 1.86 $
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -48,6 +48,7 @@
 #include "misc.h"
 #include "partimage.h"
 #include "imagefile.h"
+#include "interface.h"
 #include "interface_newt.h"
 #include "mbr_backup.h"
 
@@ -109,7 +110,7 @@ void restoreMbr(char *szImageFile, COptions *options)
     {	
       nRes = g_interface -> ErrorLogAsRoot();
       if (nRes == 2)
-        THROW( ERR_CANCELED);
+        THROW(ERR_CANCELED);
     }		
 
   // image file path
@@ -122,6 +123,11 @@ void restoreMbr(char *szImageFile, COptions *options)
 
   // ----------- get the image compression level
   options->dwCompression = (DWORD) image.getCompressionLevelForImage(szImageFile);
+  if (options->dwCompression == COMPRESS_BZIP2)
+  {
+     g_interface -> ErrorBugBzip2();
+     THROW(ERR_CANCELED);
+  }
   
   // ----------- open the image file
   image.set_options(options);
@@ -403,7 +409,18 @@ void savePartition(char *szDevice, char *szImageName, //, char *szFileSystem,
       g_interface -> askDescription(headMain.szPartDescription,
 				    MAX_DESCRIPTION);
     }
-  
+
+  // bzip2 bug workaround
+  if ((options->dwCompression == COMPRESS_BZIP2) 
+    && (!g_interface->getBatchMode()))
+    {
+      nRes = g_interface -> msgBoxContinueCancel(i18n("Bzip2 bug workaround"),
+         i18n("Because of a bug, you won't be able to restore MBR from any "
+           "bzip2 compressed image unless you manualy run bzip2 -d on them"));
+      if (nRes == MSGBOX_CANCEL)
+        THROW(ERR_CANCELED);
+    }
+ 
   // ----------- open partition file
   errno = 0;
   fDeviceFile = fopen(szDevice, "rb");
@@ -705,11 +722,22 @@ void restorePartition(char *szDevice, char *szImageName, COptions *options)
   // ----------- calculate filenames
   //image.set_dwCurrentVolumeSize(0L);
   image.set_dwVolumeNumber(0L);
-  extractFilenameFromFullPath(szImageName, aux); // filename without path
-  image.set_szOriginalFilename(aux);
-  extractFilepathFromFullPath(szImageName, aux); // filepath without filename
-  image.set_szPath(aux);
-  image.set_szImageFilename(szImageName);
+  if (strcmp(szImageName, "stdin"))
+    {
+// filename without path
+      extractFilenameFromFullPath(szImageName, aux);
+      image.set_szOriginalFilename(aux);
+// filepath without filename
+      extractFilepathFromFullPath(szImageName, aux);
+      image.set_szPath(aux);
+      image.set_szImageFilename(szImageName);
+    }
+  else
+    {
+      image.set_szOriginalFilename("stdin");
+      image.set_szPath("");
+      image.set_szImageFilename("stdin");
+    }
 
   // ----------- open partition file
   errno = 0;
@@ -876,7 +904,7 @@ void restorePartition(char *szDevice, char *szImageName, COptions *options)
 
 
   // --------- initialize info window -------------
-  guiInfo.create(szDevice, szImageName, qwPartitionSize, options->dwCompression, headMain.szOriginalDevice, headMain.szFileSystem, headMain.dateCreate, headMain.qwPartSize);
+  guiInfo.create(szDevice, szImageName, qwPartitionSize, options->dwCompression, headMain.szOriginalDevice, headMain.szFileSystem, headMain.dateCreate, headMain.qwPartSize, options);
   image.setGuiRestore(&guiInfo);
   
   /*if (image.get_bIsSplitted())
@@ -969,31 +997,38 @@ void restorePartition(char *szDevice, char *szImageName, COptions *options)
     }
   catch (CExceptions * excep)
     {
-      closeFilesRestore(true, *options, &image, fDeviceFile);
-      throw excep;
+      showDebug(1,"ERROR reading MAGIC-BEGIN-TAIL\n");
+      nRes = g_interface -> msgBoxContinueCancel(i18n("Error in Main Tail"),
+		i18n("Impossible to read the magic: the image is damaged"));
+      if (nRes != MSGBOX_CONTINUE)
+        {
+          closeFilesRestore(true, *options, &image, fDeviceFile);
+          throw excep;
+        }
     }
   
-  // check the magic string
-  /*if (strncmp(tailMain.szMagicString, MAGIC_TAIL, strlen(MAGIC_TAIL)) != 0)
-    {
-    closeFilesRestore(true, *options, &image, fDeviceFile);
-    showDebug(1, "\n*****************\n%20s\n********instead of **************\n%20s\n**************\n", tailMain.szMagicString, MAGIC_TAIL);
-    RETURN_int(-1);
-    }*/
-  
   // check CRC 64
   if (image.get_qwCRC() != tailMain.qwCRC)
     {
       showDebug(1, "CRC64 are differents: image.get_qwCRC()=%llu, tailMain.qwCRC=%llu\n",image.get_qwCRC(), tailMain.qwCRC);
       closeFilesRestore(true, *options, &image, fDeviceFile);
-      THROW(ERR_CHECK_CRC, image.get_qwCRC(), tailMain.qwCRC);
+      nRes = g_interface -> msgBoxContinueCancel(i18n("Error in Main Tail"),
+		i18n("CRC changed: the image is damaged"));
+      if (nRes != MSGBOX_CONTINUE)
+        THROW(ERR_CHECK_CRC, image.get_qwCRC(), tailMain.qwCRC);
     }
   
   // check volumes number
   if (image.get_dwVolumeNumber() != tailMain.dwVolumeNumber)
     {
-      closeFilesRestore(true, *options, &image, fDeviceFile);
-      THROW(ERR_VOLUMENUMBER, image.get_dwVolumeNumber(), tailMain.dwVolumeNumber);
+      showDebug(1, "volume numbers are differents: image.get_dwVilumeNumber()=%lu, tailMain.dwVolumeNumber=%lu\n",image.get_dwVolumeNumber(), tailMain.dwVolumeNumber);
+      nRes = g_interface -> msgBoxContinueCancel(i18n("Error in Main Tail"),
+		i18n("volume numbers changed: the image is damaged"));
+      if (nRes != MSGBOX_CONTINUE)
+        {
+          closeFilesRestore(true, *options, &image, fDeviceFile);
+          THROW(ERR_VOLUMENUMBER, image.get_dwVolumeNumber(), tailMain.dwVolumeNumber);
+        }
     }
 
   // ---- close files
@@ -1034,7 +1069,8 @@ void closeFilesRestore(bool on_error, COptions options, CImage *image, FILE *fDe
   if (fDeviceFile)
     {
       if (flock(fileno(fDeviceFile), LOCK_UN) == -1)
-        THROW(ERR_ERRNO, errno);
+	      showDebug(1, "WARNING: unlocking file failed %s\n", strerror(errno));
+//        THROW(ERR_ERRNO, errno);
       fclose(fDeviceFile);
       fDeviceFile = 0;
     }
@@ -1043,7 +1079,8 @@ void closeFilesRestore(bool on_error, COptions options, CImage *image, FILE *fDe
     {
       try { image->closeReading(); }
       catch ( CExceptions * excep)
-        { throw excep; }
+      { showDebug(1, "WARNING: closeReading failed\n"); }
+//        { throw excep; }
     }
   RETURN;
 }
@@ -1068,7 +1105,7 @@ bool isFileSystemSupported(char *szFileSystem)
       if (*szPtr)
 	szPtr++;
 
-      // comapre name
+      // compare names
       if (strncmp(szTemp, szFileSystem, sizeof(szTemp)) == 0)
 	return true;
     }
@@ -1097,33 +1134,6 @@ void lockFile(int nFileDesc, char *szDevice)
   RETURN;
 }
 
-// =======================================================
-void localDebugMsg(/*const char *szFile, const char *szFunction, int nLine, */const char *fmt, ...)
-{
-#ifdef PARTIMAGE_LOCAL_DEBUG
-  if (g_fLocalDebug) // if file opened
-    {
-      va_list args;
-      
-      va_start(args, fmt);
-      
-      /*if (getpid() == g_nDebugThreadMain) 
-	fprintf(g_fLocalDebug, "[Main] %s->%s#%d: ", szFile, szFunction, nLine);
-      else if (getpid() == g_nDebugThreadBufferR) 
-	fprintf(g_fLocalDebug, "[BufferR] %s->%s#%d: ", szFile, szFunction, nLine);
-      else if (getpid() == g_nDebugThreadBufferW) 
-	fprintf(g_fLocalDebug, "[BufferW] %s->%s#%d: ", szFile, szFunction, nLine);
-      else
-	fprintf(g_fLocalDebug, "[TH=%d] %s->%s#%d: ", getpid(), szFile, szFunction, nLine);*/
-      
-      vfprintf(g_fLocalDebug, fmt, args);
-      fflush(g_fLocalDebug);
-      
-      va_end(args);
-    }
-#endif // PARTIMAGE_LOCAL_DEBUG
-}
-
 // =======================================================
 void initCrcTable(DWORD *dwCrcTable)
 {
@@ -1270,8 +1280,6 @@ int isMounted(const char *szDevice, char *szMountPoint)
   char szBuf2 [MAXPATHLEN+1];
   char *ptrDevice;
 
-  //localDebug("\n\n====================\nbegin of isMounted[%s]\n",szDevice);
-
   // init
   bMounted = false;
   memset(szBuf1, 0, MAXPATHLEN);
@@ -1290,19 +1298,15 @@ int isMounted(const char *szDevice, char *szMountPoint)
     ptrDevice = szBuf2;
 
   showDebug(3, "isMounted(%s) = isMounted(%s)\n",szDevice, ptrDevice);
-  //localDebug("isMounted(%s) = isMounted(%s)\n",szDevice, ptrDevice);
       
   // loop: compare device with each mounted device
   while (((mnt = getmntent (f)) != 0) && (!bMounted))
     {
-      localDebug("003 entry: [%s]\n",mnt->mnt_fsname);
-
       if (strcmp(ptrDevice, mnt->mnt_fsname) == 0)
 	{
 	  bMounted = true; 
 	  strcpy(szMountPoint, mnt->mnt_dir); 
 	  showDebug(3, "partition [%s] mounted on [%s]\n",mnt->mnt_fsname,szMountPoint);
-	  //localDebug("partition [%s] mounted on [%s]\n",mnt->mnt_fsname,szMountPoint);
 	} 
     }
 
@@ -1582,11 +1586,25 @@ int getMajorMinor(char *szDevice, int *nMajor, int *nMinor)
 
   // get the major/minor number of the device
   memset(szTemp, 0, MAXPATHLEN);
-  for (i=0; (i < MAXPATHLEN) && (szDevice[i]) && (!isdigit(szDevice[i])); i++)
-    szTemp[i] = szDevice[i];
+
+  strncpy(szTemp, szDevice, MAXPATHLEN-1);
+  szTemp[MAXPATHLEN-1] = '\0';
+  i = strlen(szTemp)-1;
+  while (i && isdigit(szTemp[i]))
+    szTemp[i--] = '\0';
+
+// devfs: change part into disc
+  if (i > 3 && !strncmp("part", szTemp+i-3, 4)) 
+    strcpy(szTemp+i-3, "disc");
+
+  showDebug(1, "getMajorMinor: device=%s\n", szTemp);
+
   nRes = stat(szTemp, &fStat);
   if (nRes == -1)
-    RETURN_int(-1);
+    {
+      showDebug(1, "stat failed: %d\n", errno);
+      RETURN_int(-1);
+    }
   
   *nMajor = major(fStat.st_rdev);
   *nMinor = minor(fStat.st_rdev); // it is often 64 (for hdb, hdd, ...) or 0 (hda, hdc, ...)
@@ -1674,6 +1692,46 @@ int bigFseek(FILE *fStream, QWORD qwOffset) // SEEK_SET forced
   return 0;
 }
 
+/*
+ * -1: fatal error
+ *  1: devfs mounted
+ *  0: devfs not mounted
+ */
+
+int isDevfsMounted()
+{
+	FILE *f;
+	char buf[256];
+
+	if (access("/proc/mounts",R_OK)==-1)
+	{
+		g_interface->msgBoxError(i18n("You haven't access to /proc/mounts."));
+		return(-1);
+	}
+
+	if ((f=fopen("/proc/mounts","r"))==NULL)
+	{
+		g_interface->msgBoxError(i18n("An error occurred trying to read /proc/mounts."));
+		return(-1);
+	}
+
+	while (!feof(f))
+	{
+		fscanf(f,"%255s",buf); *(buf+255)='\0';
+		if (!strcmp(buf,"devfs"))
+		{
+			fclose(f);
+			return(1);
+		}
+	}
+
+	fclose(f);
+	return(0);
+}
+
+
+
+
 // =======================================================
 int isDevfsEnabled()
 {
@@ -1712,13 +1770,16 @@ int isDevfsEnabled()
 }
 
 // =======================================================
-char *decodePartitionEntry(char *cPtr, int *nMajor, int *nMinor, int *nBlocks, int *nPartNum, char *szDevice)
+char *decodePartitionEntry(char *cPtr, int *nMajor, int *nMinor,
+   int *nBlocks, int *nPartNum, char *szDevice)
 {
   char cTemp[128];
   char *cCur;
   char *cDevPtr1;
   char *cDevPtr2;
-  
+
+  showDebug(9, "start decoding entry\n");
+ 
   // Major number
   while (!isalnum(*cPtr))
     cPtr++;
@@ -1796,6 +1857,9 @@ char *decodePartitionEntry(char *cPtr, int *nMajor, int *nMinor, int *nBlocks, i
     }
   *nPartNum = atoi(cTemp);
   
+  showDebug(9, "entry found: %s %d %d %d %d\n", szDevice, nMajor, nMinor,
+    nBlocks, nPartNum); 
+ 
   return cPtr;
 }
 
