@@ -41,7 +41,6 @@
 #define RETURN {showDebug(9, "end\n"); return;}
 
 #define showDebug(level, format, args...) showDebugMsg(g_fDebug, level, __FILE__, __FUNCTION__, __LINE__, format, ## args)
-#define localDebug(format, args...) localDebugMsg(format, ## args)
 #define THROW(nError, args...) showDebug(1, "THROW: %d\n", nError),throw new CExceptions(__FUNCTION__, nError, ## args)
 #define SNPRINTF(szDest, szFormat, args...) snprintf(szDest, sizeof(szDest)-1, szFormat, ## args),szDest[sizeof(szDest)-1]=0 // allow to use this macro in 'else'
 
@@ -69,8 +68,8 @@ extern FILE *g_fLocalDebug;
 extern char g_szDebug[];
 
 extern DWORD g_dwCrcTable[];
-class CInterfaceNewt;
-extern CInterfaceNewt *g_interface;
+class CInterface;
+extern CInterface *g_interface;
 extern bool g_bSigInt;
 extern bool g_bSigKill;
 
@@ -80,10 +79,15 @@ extern bool g_bSigKill;
   #define i18n(X) (X)
 #endif
 
-#define min(a,b) (a < b ? a : b)
-#define min3(a,b,c) (min(min(a,b),c))
-#define max(a,b) (a > b ? a : b)
-#define max3(a,b,c) (max(max(a,b),c))
+#define my_min(a,b) ((a) < (b) ? (a) : (b))
+#define my_max(a,b) ((a) > (b) ? (a) : (b))
+
+/*
+#define min(a,b) ((a) < (b) ? (a) : (b))
+#define min3(a,b,c) (min(min((a),(b)),(c)))
+#define max(a,b) ((a) > (b) ? (a) : (b))
+#define max3(a,b,c) (max(max((a),(b)),(c)))
+*/
 
 #define OPERATION_SAVE       1
 #define OPERATION_RESTORE    2
@@ -125,6 +129,9 @@ extern bool g_bSigKill;
 #define MAX_HOSTNAMESIZE	128
 #define MAX_DEVICENAMELEN       512
 
+#define MAX_USERNAMELEN		32
+#define MAX_PASSWORDLEN		32
+
 // COMPRESSION
 #define COMPRESS_NONE    0
 #define COMPRESS_GZIP    1
@@ -143,6 +150,10 @@ extern bool g_bSigKill;
 #define FINISH_HALT      1
 #define FINISH_REBOOT    2
 #define FINISH_QUIT      3
+#define FINISH_LAST      4
+
+#define FINISH_LAST_COUNTFILE "/tmp/partimage.count"
+#define FINISH_LAST_COUNTFILE_LOCK "/tmp/partimage.count.lock"
 
 // ===================== MAGIC STRINGS  ============================
 #define MAGIC_BEGIN_LOCALHEADER	              "MAGIC-BEGIN-LOCALHEADER"
@@ -175,6 +186,7 @@ extern bool g_bSigKill;
 #define OPTIONS_DEFAULT_SERVERPORT	      4025
 #define OPTIONS_DEFAULT_AUTOSTART             false
 #define OPTIONS_DEFAULT_SPLIT_WAIT            false
+#define OPTIONS_DEFAULT_RUN_SHELL             false
 #define OPTIONS_DEFAULT_SYNC                  true
 #define OPTIONS_DEFAULT_SSL                   true
 #define OPTIONS_DEFAULT_BACKUP_MBR            true
@@ -182,6 +194,9 @@ extern bool g_bSigKill;
 #define OPTIONS_DEFAULT_SERVERNAME	      ""
 #define OPTIONS_DEFAULT_AUTOMOUNT             ""
 
+#define OPTIONS_DEFAULT_USERNAME			  ""
+#define OPTIONS_DEFAULT_PASSWORD			  ""
+
 // ======================== FUNCTIONS ================================
 void usage();
 int checkStructSizes();
