@@ -5,7 +5,7 @@
     copyright            : (C) 2000 by Franck Ladurelle
     email                : ladurelf@partimage.org
  ***************************************************************************/
-// $Revision: 1.18 $
+// $Revision: 1.24 $
 /***************************************************************************
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -15,6 +15,9 @@
  *                                                                         *
  ***************************************************************************/
 
+// PAM support is writen based on code from Chris Evans for
+// Very Secure FTPd.
+
 #ifdef HAVE_CONFIG_H
   #include <config.h>
 #endif
@@ -23,11 +26,15 @@
 #include <stdlib.h>
 
 // shadow.h and crypt.h may not exist on bsd
-#ifdef HAVE_SHADOW_H
-  #include <shadow.h>
-#endif
-#ifdef HAVE_CRYPT_H
-  #include <crypt.h>
+#ifdef HAVE_PAM
+  #include <security/pam_appl.h>
+#else
+  #ifdef HAVE_SHADOW_H
+    #include <shadow.h>
+  #endif
+  #ifdef HAVE_CRYPT_H
+    #include <crypt.h>
+  #endif
 #endif
 
 #include <time.h>
@@ -42,6 +49,7 @@
 
 extern CPrivs * g_privs;
 
+#ifndef HAVE_PAM
 char * GetSalt(char * szLogin)
 {
 #ifdef OS_LINUX
@@ -64,40 +72,50 @@ char * GetSalt(char * szLogin)
       r = random() % 52;
       *(salt+i) = (r>25 ? 'a'+r-26 : 'A'+r);
     }
- 
-  g_privs->Root(); 
-#ifdef OS_LINUX
-  s_pass = getspnam(szLogin);
-#endif
-#ifdef OS_FBSD
-  s_pass = getpwnam(szLogin);
-#endif
-  g_privs->User(); 
 
-#ifdef OS_LINUX
-  if (s_pass && strlen(s_pass->sp_pwdp) > 10)
-    memcpy(salt, s_pass->sp_pwdp, 11);
-  else if (s_pass)
-    memcpy(salt, s_pass->sp_pwdp, strlen(s_pass->sp_pwdp));
-#endif
-#ifdef OS_FBSD
-  if (s_pass && strlen(s_pass->pw_passwd) > 10)
-    memcpy(salt, s_pass->pw_passwd, 11);
-  else if (s_pass)
-    memcpy(salt, s_pass->pw_passwd, strlen(s_pass->pw_passwd));
-#endif
+#ifdef MUST_LOGIN
+    g_privs->Root(); 
+  #ifdef OS_LINUX
+    s_pass = getspnam(szLogin);
+  #endif
+  #ifdef OS_FBSD
+    s_pass = getpwnam(szLogin);
+  #endif
+    g_privs->User(); 
+
+  #ifdef OS_LINUX
+    if (s_pass && strlen(s_pass->sp_pwdp) > 10)
+      memcpy(salt, s_pass->sp_pwdp, 11);
+    else if (s_pass)
+      memcpy(salt, s_pass->sp_pwdp, strlen(s_pass->sp_pwdp));
+  #endif
+  #ifdef OS_FBSD
+    if (s_pass && strlen(s_pass->pw_passwd) > 10)
+      memcpy(salt, s_pass->pw_passwd, 11);
+    else if (s_pass)
+      memcpy(salt, s_pass->pw_passwd, strlen(s_pass->pw_passwd));
+  #endif
+#else  // MUST_LOGIN
+  *salt = '\0';
+#endif // MUST_LOGIN
 
   showDebug(9, "end of access::getsalt\n");
-  return strdup(salt);
+  return salt;
 }
+#else // HAVE_PAM
+char * GetSalt(char * szLogin)
+{
+  return "";
+}
+#endif
 
 #ifdef MUST_LOGIN  
+#ifndef HAVE_PAM
 unsigned int CheckAccess(bool bMustLogin, char * szLogin, char * szPasswd)
 {
   if (!bMustLogin)
     RETURN_int(0)
 
-
   FILE * f;
 #ifdef OS_LINUX
   struct spwd * s_pass;
@@ -110,8 +128,6 @@ unsigned int CheckAccess(bool bMustLogin, char * szLogin, char * szPasswd)
   char * ptr, * ptr2;
   BEGIN;
 
-
-
   f = fopen(PARTIMAGED_USERS, "r");
   if (!f) 
     {
@@ -184,15 +200,106 @@ unsigned int CheckAccess(bool bMustLogin, char * szLogin, char * szPasswd)
   else
     RETURN_int(10);
 #endif
-
 }
-#else
-unsigned int CheckAccess(bool b, char * szLogin, char * szPasswd)
+#else // HAVE_PAM
+static char * g_szPassword;
+int pam_conv_partimaged(int nmsg, const struct pam_message ** p_messages,
+   struct pam_response ** p_responses, void * p_adddata)
+{
+  struct pam_response * response;
+  
+  response = (struct pam_response *) malloc (sizeof(struct pam_response)*nmsg);
+  if (!response)
+    {
+      showDebug(2, "PAM: cannot allocate memory for response\n");
+      return PAM_CONV_ERR;
+    }
+  showDebug(1, "PAM: conv called\n");
+  for (int i=0; i < nmsg; ++i)
+    switch (p_messages[i] -> msg_style)
+    {
+      case PAM_PROMPT_ECHO_OFF:   // send user password to PAM module
+        showDebug(1, "PAM ECHOOFF\n");
+        response[i].resp_retcode = PAM_SUCCESS;
+        response[i].resp = strdup(g_szPassword);
+        break;
+
+      case PAM_PROMPT_ECHO_ON:    // we don't want this
+        showDebug(2, "PAM: PROMPT_ECHO_ON refused by us.\n"); 
+        free(response);
+        return PAM_CONV_ERR;
+
+      case PAM_TEXT_INFO:
+      case PAM_ERROR_MSG:
+        showDebug(2, "PAM message: %s\n", p_messages[i] -> msg); 
+        response[i].resp_retcode = PAM_SUCCESS; 
+        response[i].resp = NULL;
+        break;
+
+      default:
+        showDebug(2, "PAM: wrong message code: %d.\n",
+           p_messages[i] -> msg_style); 
+        free(response);
+        return PAM_CONV_ERR;
+    }
+  *p_responses = response;  // send answers for messages to PAM module
+  return PAM_SUCCESS; 
+}
+
+unsigned int CheckAccess(bool bMustLogin, char * szLogin, char * szPasswd)
+{
+  int nRes;
+  pam_handle_t * pamh;
+  struct pam_conv partimaged_conv = { &pam_conv_partimaged, 0 };
+
+  if (!bMustLogin)
+    RETURN_int(0)
+
+  g_szPassword = strdup(szPasswd);
+
+  nRes = pam_start("partimaged", szLogin, &partimaged_conv, &pamh);
+  if (nRes != PAM_SUCCESS)
+    {
+      pam_end(pamh, nRes);
+      showDebug(1, "pam_start failed: %s\n", 
+         pam_strerror(pamh, nRes));
+      return 1;
+    }
+ 
+  g_privs->Root(); 
+  nRes = pam_authenticate(pamh, 0);
+  g_privs->User(); 
+  if (nRes != PAM_SUCCESS)
+    {
+      pam_end(pamh, nRes);
+      showDebug(1, "pam_autenticate failed: %s\n", 
+         pam_strerror(pamh, nRes));
+      return 2;
+    }
+  
+  g_privs->Root(); 
+  nRes = pam_acct_mgmt(pamh, 0);
+  g_privs->User(); 
+  if (nRes != PAM_SUCCESS)
+    {
+      pam_end(pamh, nRes);
+      showDebug(1, "pam_acct_mgmt failed: %s\n", 
+         pam_strerror(pamh, nRes));
+      return 3;
+    }
+  
+  nRes = pam_end(pamh, nRes);
+  return (nRes == PAM_SUCCESS ? 0 : 4);
+}
+#endif  // HAVE_PAM
+#else   // MUST_LOGIN
+unsigned int CheckAccess(bool bMustLogin, char * szLogin, char * szPasswd)
 {
   RETURN_int(0);
 }
 #endif
 
+#ifndef HAVE_PAM
 static void help(char * szFile)
 {
   fprintf(stderr,"error when verifying %s, check logfile for help\n",
@@ -201,7 +308,6 @@ static void help(char * szFile)
   fprintf(stderr,"if you don't have logfile, use partimaged --debug=1\n");
 }
 
-
 unsigned int CheckAccessFile(char * szFile)
 {
   int nRes;
@@ -243,3 +349,9 @@ unsigned int CheckAccessFile(char * szFile)
   
   return 0;
 }
+#else // HAVE_PAM
+unsigned int CheckAccessFile(char * szFile)
+{
+  return 0;
+}
+#endif
