@@ -73,6 +73,8 @@ void formatCompilOptions(char *szDest, int nMaxLen)
   memset(szDest, 0, nMaxLen);
 
   addText(i18n("==== Partition Image: compilation options used ====\n"));
+  addText(i18n("\n* Version is %s [%s].\n"), PACKAGE_VERSION, __VERSION__);
+
   addText(i18n("\n* Supported file systems:\n  - "));
   addText(g_szSupportedFileSystems);
   addText(i18n("\n"));
@@ -85,9 +87,6 @@ void formatCompilOptions(char *szDest, int nMaxLen)
 
   addText(i18n("\n* Debug options:\n"));
 
-#ifdef PARTIMAGE_LOCAL_DEBUG
-  addText(i18n("  - PARTIMAGE_LOCAL_DEBUG=%s\n"), PARTIMAGE_LOCAL_DEBUG);
-#endif
 #ifdef PARTIMAGE_LOG
   addText(i18n("  - PARTIMAGE_LOG=%s\n"), PARTIMAGE_LOG);
 #endif
@@ -257,22 +256,22 @@ char *formatSize2(QWORD qwSize, char *szText, int nMaxLen)
   else if (qwSize < llMegaB) // In KiloBytes
     {
       dSize = ((double) qwSize) / ((double) 1024.0);
-      snprintf(szText, nMaxLen, i18n("%.2f KB"), (float) dSize);
+      snprintf(szText, nMaxLen, i18n("%.2f KiB"), (float) dSize);
     }
   else if (qwSize < llGigaB) // In MegaBytes
     {
       dSize = ((double) qwSize) / ((double) (1024.0 * 1024.0));
-      snprintf(szText, nMaxLen, i18n("%.2f MB"), (float) dSize);
+      snprintf(szText, nMaxLen, i18n("%.2f MiB"), (float) dSize);
     }
   else if (qwSize < llTeraB)// In GigaBytes
     {
       dSize = ((double) qwSize) / ((double) (1024.0 * 1024.0 * 1024.0));
-      snprintf(szText, nMaxLen, i18n("%.2f GB"), (float) dSize);
+      snprintf(szText, nMaxLen, i18n("%.2f GiB"), (float) dSize);
     }
   else // In TeraBytes
     {
       dSize = ((double) qwSize)/((double) (1024.0 * 1024.0 * 1024.0 * 1024.0));
-      snprintf(szText, nMaxLen, i18n("%.2f TB"), (float) dSize);
+      snprintf(szText, nMaxLen, i18n("%.2f TiB"), (float) dSize);
     }
   
   return szText;
