@@ -103,7 +103,7 @@ struct CVolumeHeader // size must be 512 (adjust the reserved data)
   QWORD qwIdentificator; // to avoid using volumes from differents image files on restore
   
   unsigned char cReserved[404]; // Adjust to fit with total header size
-};
+} __attribute__ ((packed));
 
 // ===================================================================
 struct CMainTail // size must be 16384 (adjust the reserved data)
@@ -120,6 +120,7 @@ struct COptions
   bool bOverwrite;
   //DWORD dwSplitMode;
   bool bSplitWait; // Wait after each volume change
+  bool bRunShell;  // run a shell command on each volume change
   bool bEraseWithNull;
   bool bCheckBeforeSaving;
   bool bFailIfMounted; // Fail if file system is mounted
@@ -136,6 +137,9 @@ struct COptions
   QWORD qwSplitSize; // 0L = auto, !0 = bytes size
   char szServerName[MAX_HOSTNAMESIZE]; // partimaged server's name
   char szAutoMount[MAXPATHLEN];
+  char szUserName[MAX_USERNAMELEN];
+  char szPassWord[MAX_PASSWORDLEN];
+  char szFullyBatchMode[2048];
 };
 
 #define MAX_UNAMEINFOLEN 65 //SYS_NMLN
@@ -146,7 +150,7 @@ struct CMainHeader // size must be 16384 (adjust the reserved data)
   char szFileSystem[512]; // ext2fs, ntfs, reiserfs, ...
   char szPartDescription[MAX_DESCRIPTION]; // user description of the partition
   char szOriginalDevice[MAX_DEVICENAMELEN]; // original partition name
-  char szFirstImageFilepath[MAXPATHLEN]; // for splitted image files
+  char szFirstImageFilepath[4095]; //MAXPATHLEN]; // for splitted image files
   
   // system and hardware infos
   char szUnameSysname[MAX_UNAMEINFOLEN];
@@ -183,5 +187,5 @@ struct CMainHeader // size must be 16384 (adjust the reserved data)
   DWORD dwReservedFuture009;
 
   BYTE cReserved[6524]; // Adjust to fit with total header size
-};
+} __attribute__ ((packed));
 #endif //COMMON_H
