@@ -247,11 +247,12 @@ DWORD CImageDisk::read(void *buf, DWORD dwLength) // [Buffer-Thread]
 
   if (dwRes < dwLength)
     {
-      showDebug(1, "EOF: dwRes=%lu and dwLength=%lu\n", dwRes, dwLength);
+      showDebug(1, "EOF: dwRes=%lu and dwLength=%lu: MISSING: %lu\n", dwRes, dwLength, dwLength-dwRes);
     }
 
   m_qwTotal += (QWORD) dwRes; //dwLength;
-  //showDebug(1, "m_qwTotal = %lld\n", m_qwTotal);
+  showDebug(1, "cid: %ld ; m_qwTotal = %lld\n", dwRes, m_qwTotal);
+
   return dwRes; // important: must return dwRes even if EOF reached
 }
 
@@ -297,6 +298,9 @@ int CImageDisk::getCompressionLevelForImage(char *szFilename) // [Main-Thread]
 
   BEGIN;
 
+  if (!strcmp(szFilename, "stdin"))
+    return COMPRESS_NONE;
+
   gzFile gzImageFile;
   FILE *fImageFile;
   BZFILE *bzImageFile;
@@ -379,57 +383,65 @@ void CImageDisk::openReading() // [Main-Thread]
 
   BEGIN;
   showDebug(1, "ALREADY=%llu\n", m_qwTotal);
+  m_qwTotal=0LL;
   if (m_bIsOpened == true)
     THROW(ERR_OPENED);
 
-  // check the file exists
-  errno = 0;
-  if (access(m_szImageFilename, F_OK) == -1) // if the file doesn't exists
-    THROW(ERR_NOTFOUND);
-  
-  showDebug(1, "openReading[%s]\n", m_szImageFilename);
-
-#ifdef OS_LINUX
-  #ifdef HAVE_OPEN64
-    m_nImageFd = open64(m_szImageFilename, O_RDONLY | O_LARGEFILE | O_NOFOLLOW);
-  #else
-    m_nImageFd = open(m_szImageFilename, O_RDONLY | O_LARGEFILE | O_NOFOLLOW);
-  #endif
-#endif
-#ifdef OS_FBSD
-  m_nImageFd = open(m_szImageFilename, O_RDONLY | O_NOFOLLOW);
-#endif
-  if (m_nImageFd == -1)
-    THROW(ERR_ERRNO, errno);
-  else 
-    showDebug(1, "preopen successfull for %s\n", m_szImageFilename);
-
-  // ---- avoid RACE CONDITIONS security problems:
-  errno = 0;
-#ifdef HAVE_FSTAT64
-  if (fstat64(m_nImageFd, &fStat) == -1)
-#else
-  if (fstat(m_nImageFd, &fStat) == -1)
-#endif
+  if (strcmp(m_szImageFilename, "stdin"))
     {
-      showDebug(1, "fstat() returned -1 and errno=%d=[%s]\n", errno, strerror(errno));
-      THROW(ERR_ERRNO, errno);
-    }
+      // check the file exists
+      errno = 0;
+      if (access(m_szImageFilename, F_OK) == -1) // if the file doesn't exists
+        THROW(ERR_NOTFOUND);
+  
+      showDebug(1, "openReading[%s]\n", m_szImageFilename);
 
-  if (!S_ISREG(fStat.st_mode)) // not a regular file
-    THROW(ERR_NOTAREGULARFILE, m_szImageFilename); //BIGH
+    #ifdef OS_LINUX
+      #ifdef HAVE_OPEN64
+        m_nImageFd = open64(m_szImageFilename, O_RDONLY | O_LARGEFILE | O_NOFOLLOW);
+      #else
+        m_nImageFd = open(m_szImageFilename, O_RDONLY | O_LARGEFILE | O_NOFOLLOW);
+      #endif
+    #endif
+    #ifdef OS_FBSD
+      m_nImageFd = open(m_szImageFilename, O_RDONLY | O_NOFOLLOW);
+    #endif
+      if (m_nImageFd == -1)
+        THROW(ERR_ERRNO, errno);
+      else 
+        showDebug(1, "preopen successfull for %s\n", m_szImageFilename);
+
+      // ---- avoid RACE CONDITIONS security problems:
+      errno = 0;
+    #ifdef HAVE_FSTAT64
+      if (fstat64(m_nImageFd, &fStat) == -1)
+    #else
+      if (fstat(m_nImageFd, &fStat) == -1)
+    #endif
+        {
+          showDebug(1, "fstat() returned -1 and errno=%d=[%s]\n", errno, strerror(errno));
+          THROW(ERR_ERRNO, errno);
+        }
+
+      if (!S_ISREG(fStat.st_mode)) // not a regular file
+        THROW(ERR_NOTAREGULARFILE, m_szImageFilename); //BIGH
   
-  // ---- lock the image file
-  if (lockImage(LOCK_SH, m_nImageFd, m_szImageFilename) == -1)
-    THROW(ERR_LOCKED, m_szImageFilename);
+      // ---- lock the image file
+      if (lockImage(LOCK_SH, m_nImageFd, m_szImageFilename) == -1)
+        THROW(ERR_LOCKED, m_szImageFilename);
   
-  m_fImageFile = fdopen(m_nImageFd, "rb");
-  if (m_fImageFile == NULL)
-    THROW( errno);
+      m_fImageFile = fdopen(m_nImageFd, "rb");
+      if (m_fImageFile == NULL)
+        THROW( errno);
+      else
+        showDebug(1, "open\n");
+      showDebug(1, "file %s opened ok\n", m_szImageFilename);
+    } // stdin?
   else
-    showDebug(1, "open\n");
-
-  showDebug(1, "%s opened ok\n", m_szImageFilename);
+    { 
+      m_fImageFile = stdin;
+      showDebug(1, "stdin opened for reading\n");
+    }
 	
   m_bIsOpened = true;
 }	
@@ -517,67 +529,86 @@ void CImageDisk::openWriting() // [Main-Thread]
   snprintf(m_szSpaceFilename, MAXPATHLEN, "%s/%8s.tmp", szPathDir, szShortName);
   showDebug(1, "TEMP=[%s]\n", m_szSpaceFilename);
 
-  errno = 0;
-  nRes = createSpaceFile();
-  if (nRes == -1)
-    {
-      if (errno == ENOSPC)
-	{
-	  showDebug(1, "No space left -> cannot create file [%s]\n",
-             m_szSpaceFilename);
-	  THROW(ERR_CREATESPACEFILENOSPC, m_szSpaceFilename);
-	}
-      else
-	{
-	  showDebug(1, "Cannot create SpaceFile[%s]\n", m_szSpaceFilename); 
-	  THROW(ERR_CREATESPACEFILE, m_szSpaceFilename);
-	}
+  if (strcmp(m_szImageFilename, "stdout"))
+    { // we create spacefile only if imagefile is not stdout
+      errno = 0;
+      nRes = createSpaceFile();
+      if (nRes == -1)
+        {
+          if (errno == ENOSPC)
+    	    {
+	      showDebug(1, "No space left -> cannot create file [%s]\n",
+                 m_szSpaceFilename);
+	      THROW(ERR_CREATESPACEFILENOSPC, szPathDir);
+    	    }
+          else if (errno == EACCES)
+            {
+              showDebug(1, "Permission denied -> cannot create file [%s]\n",
+                 m_szSpaceFilename);
+	      THROW(ERR_CREATESPACEFILEDENIED, szPathDir);
+    	    }
+          else
+	    {
+	      showDebug(1, "Cannot create SpaceFile[%s]\n", m_szSpaceFilename); 
+	      THROW(ERR_CREATESPACEFILE, m_szSpaceFilename);
+	    }
+        }
     }
 
-  errno = 0;
-  flags = O_CREAT | O_WRONLY | O_NOFOLLOW;
-#ifdef OS_LINUX
-  flags |= O_LARGEFILE;
-#endif
-  if (m_options.bOverwrite != true)
-    flags |= O_EXCL;
-
-#ifdef HAVE_OPEN64
-  m_nImageFd = open64(m_szImageFilename, flags, S_IREAD | S_IWRITE);
-#else
-  m_nImageFd = open(m_szImageFilename, flags, S_IREAD | S_IWRITE);
-#endif
-  if ((m_nImageFd == -1))
-    if (errno == EEXIST)
-      THROW(ERR_EXIST);
-    else
-      THROW(ERR_ERRNO, errno);
-
-  // ---- avoid RACE CONDITIONS security problems:
-#ifdef HAVE_FSTAT64
-  if (fstat64(m_nImageFd, &fStat) == -1)
-#else
-  if (fstat(m_nImageFd, &fStat) == -1)
-#endif
-    THROW(ERR_ERRNO, errno);
-  
-  if (!S_ISREG(fStat.st_mode)) // not a regular file
-    THROW(ERR_NOTAREGULARFILE, m_szImageFilename);
-
-  // lock the image file
-  if (lockImage(LOCK_EX, m_nImageFd, m_szImageFilename) == -1)
-    THROW(ERR_LOCKED, m_szImageFilename);
-
-  // truncate the file (now, we can because it's not a symlink)
-  if (ftruncate(m_nImageFd, 0L) == -1)
-    THROW(ERR_ERRNO, errno);
-
-  showDebug(1, "open std\n");
-  m_fImageFile = fdopen(m_nImageFd, "wb");
-  if (m_fImageFile == NULL)
+  if (strcmp(m_szImageFilename, "stdout"))
     {
-      showDebug(1, "error:%d %s\n", errno, strerror(errno));
-      THROW(ERR_ERRNO, errno);
+      errno = 0;
+      flags = O_CREAT | O_WRONLY | O_NOFOLLOW;
+    #ifdef OS_LINUX
+      flags |= O_LARGEFILE;
+    #endif
+      if (m_options.bOverwrite != true)
+        flags |= O_EXCL;
+
+    #ifdef HAVE_OPEN64
+      m_nImageFd = open64(m_szImageFilename, flags, S_IREAD | S_IWRITE);
+    #else
+      m_nImageFd = open(m_szImageFilename, flags, S_IREAD | S_IWRITE);
+    #endif
+      if ((m_nImageFd == -1))
+        if (errno == EEXIST)
+          THROW(ERR_EXIST);
+        else
+          THROW(ERR_ERRNO, errno);
+
+      // ---- avoid RACE CONDITIONS security problems:
+    #ifdef HAVE_FSTAT64
+      if (fstat64(m_nImageFd, &fStat) == -1)
+    #else
+      if (fstat(m_nImageFd, &fStat) == -1)
+    #endif
+        THROW(ERR_ERRNO, errno);
+  
+      if (!S_ISREG(fStat.st_mode)) // not a regular file
+        THROW(ERR_NOTAREGULARFILE, m_szImageFilename);
+
+      // lock the image file
+      if (lockImage(LOCK_EX, m_nImageFd, m_szImageFilename) == -1)
+        THROW(ERR_LOCKED, m_szImageFilename);
+
+/* FIXME: this fails when using LUFS. Is it required?
+      // truncate the file (now, we can because it's not a symlink)
+      if (ftruncate(m_nImageFd, 0L) == -1)
+        THROW(ERR_ERRNO, errno);
+*/
+
+      showDebug(1, "open std\n");
+      m_fImageFile = fdopen(m_nImageFd, "wb");
+      if (m_fImageFile == NULL)
+        {
+          showDebug(1, "error:%d %s\n", errno, strerror(errno));
+          THROW(ERR_ERRNO, errno);
+        }
+    }
+  else // it's stdout
+    {
+      m_fImageFile = stdout;
+      showDebug(1, "image will be on stdout\n");
     }
 
   m_bIsOpened = true;
@@ -658,11 +689,17 @@ bool CImageDisk::doesFileExists(char *szPath) // [Main-Thread]
   int nRes;
 
   errno = 0;
+  if ( strcmp(szPath, "stdin") ) {
 #ifdef HAVE_OPEN64
   nFd = open64(szPath, O_RDONLY | O_NOFOLLOW | O_LARGEFILE);
 #else
   nFd = open(szPath, O_RDONLY | O_NOFOLLOW | O_LARGEFILE);
 #endif
+  } else { // it's stdin
+  nFd=0;
+  return true;
+  }
+  showDebug(3, "doesFileExists(%s) =>%d\n",szPath,nFd);
   //fStream = fopen(szPath, "rb");
   //if (!fStream) // error
   if (nFd == -1)
@@ -739,15 +776,24 @@ int CImageDisk::mountImageLocation(char * szMountDevice, char * szMountPoint,
   int nRes;
   
   errno = 0;
+  showDebug(1, "automount-> %s %s %s\n", szMountDevice, szMountPoint,szMountFS);
 #ifdef OS_LINUX
-  nRes = mount(szMountDevice, szMountPoint, szMountFS, 0, NULL);
+  if (!strcasecmp(szMountFS, "iso9660"))
+    nRes = mount(szMountDevice, szMountPoint, szMountFS, MS_RDONLY, NULL);
+  else if (!strcasecmp(szMountFS, "udf"))
+    nRes = mount(szMountDevice, szMountPoint, szMountFS, MS_NOATIME, NULL);
+  else
+    nRes = mount(szMountDevice, szMountPoint, szMountFS, 0, NULL);
 #endif
 #ifdef OS_FBSD
   nRes = mount(szMountFS, szMountPoint, 0, NULL);
 #endif
 
   if ((nRes) && (errno != EBUSY))  // EBUSY==already mounted
-    return -1;
+    {
+      showDebug(1, "mount failed: %d (%s)\n", errno, strerror(errno));
+      return -1;
+    }
   else
     return 0;
 }  
